from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from navigation import Link

from .permissions import permission_acl_view, permission_acl_edit


def get_kwargs_factory(variable_name):
    def get_kwargs(context):
        content_type = ContentType.objects.get_for_model(
            context[variable_name]
        )
        return {
            'app_label': '"{}"'.format(content_type.app_label),
            'model': '"{}"'.format(content_type.model),
            'object_id': '{}.pk'.format(variable_name)
        }

    return get_kwargs


link_acl_delete = Link(
    permissions=(permission_acl_edit,), tags='dangerous', text=_('Delete'),
    view='acls:acl_delete', args='resolved_object.pk'
)
link_acl_list = Link(
    permissions=(permission_acl_view,), text=_('ACLs'), view='acls:acl_list',
    kwargs=get_kwargs_factory('resolved_object')
)
link_acl_new = Link(
    permissions=(permission_acl_edit,), text=_('New ACL'),
    view='acls:acl_new', kwargs=get_kwargs_factory('resolved_object')
)
link_acl_permissions = Link(
    permissions=(permission_acl_edit,), text=_('Permissions'),
    view='acls:acl_permissions', args='resolved_object.pk'
)



from __future__ import unicode_literals

from django.conf import settings
from django.conf.urls import include, patterns, url
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^docs/', include('rest_framework_swagger.urls')),
)


def handler500(request):
    """
    500 error handler which includes ``request`` in the context.

    Templates: `500.html`
    Context: None
    """
    from django.template import Context, loader
    from django.http import HttpResponseServerError

    t = loader.get_template('500.html')  # You need to create a 500.html template.
    return HttpResponseServerError(t.render(Context({
        'request': request,
    })))

if settings.DEBUG:
    from django.conf.urls.static import static
    urlpatterns += static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)

    if 'rosetta' in settings.INSTALLED_APPS:
        urlpatterns += patterns(
            '',
            url(r'^rosetta/', include('rosetta.urls'), name='rosetta'))

    if 'debug_toolbar' in settings.INSTALLED_APPS:
        import debug_toolbar
        urlpatterns += patterns(
            '',
            url(r'^__debug__/', include(debug_toolbar.urls)))



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from .exceptions import NewDocumentVersionNotAllowed
from .models import DocumentCheckout


def check_if_new_versions_allowed(sender, **kwargs):
    if not DocumentCheckout.objects.are_document_new_versions_allowed(kwargs['instance'].document):
        raise NewDocumentVersionNotAllowed(
            _(
                'New versions not allowed for the checkedout document: %s'
                % kwargs['instance'].document
            )
        )



from __future__ import unicode_literals

from django.core.files.base import File
from django.core.exceptions import ValidationError
from django.test import TestCase, override_settings

from documents.models import DocumentType
from documents.tests import TEST_SMALL_DOCUMENT_PATH, TEST_DOCUMENT_TYPE

from ..models import MetadataType, DocumentMetadata

TEST_DEFAULT_VALUE = 'test'
TEST_LOOKUP_TEMPLATE = '1,2,3'
TEST_INCORRECT_LOOKUP_VALUE = '0'
TEST_CORRECT_LOOKUP_VALUE = '1'
TEST_DATE_VALIDATOR = 'metadata.validators.DateValidator'
TEST_DATE_PARSER = 'metadata.parsers.DateParser'
TEST_INVALID_DATE = '___________'
TEST_VALID_DATE = '2001-1-1'
TEST_PARSED_VALID_DATE = '2001-01-01'


@override_settings(OCR_AUTO_OCR=False)
class MetadataTestCase(TestCase):
    def setUp(self):
        self.metadata_type = MetadataType.objects.create(
            name='test', label='test'
        )

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        self.document_type.metadata.create(metadata_type=self.metadata_type)

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()
        self.metadata_type.delete()

    def test_no_default(self):
        document_metadata = DocumentMetadata(
            document=self.document, metadata_type=self.metadata_type
        )

        document_metadata.full_clean()
        document_metadata.save()

        self.assertEqual(self.document.metadata_value_of.test, None)

    def test_default(self):
        self.metadata_type.default = TEST_DEFAULT_VALUE
        self.metadata_type.save()

        document_metadata = DocumentMetadata(
            document=self.document, metadata_type=self.metadata_type
        )

        document_metadata.full_clean()
        document_metadata.save()

        self.assertEqual(self.document.metadata_value_of.test, TEST_DEFAULT_VALUE)

    def test_lookup(self):
        self.metadata_type.lookup = TEST_LOOKUP_TEMPLATE

        document_metadata = DocumentMetadata(
            document=self.document, metadata_type=self.metadata_type, value=TEST_INCORRECT_LOOKUP_VALUE
        )

        with self.assertRaises(ValidationError):
            # Should return error
            document_metadata.full_clean()
            document_metadata.save()

        # Should not return error
        document_metadata.value = TEST_CORRECT_LOOKUP_VALUE
        document_metadata.full_clean()
        document_metadata.save()

        self.assertEqual(self.document.metadata_value_of.test, TEST_CORRECT_LOOKUP_VALUE)

    def test_validation(self):
        self.metadata_type.validation = TEST_DATE_VALIDATOR

        document_metadata = DocumentMetadata(
            document=self.document, metadata_type=self.metadata_type, value=TEST_INVALID_DATE
        )

        with self.assertRaises(ValidationError):
            # Should return error
            document_metadata.full_clean()
            document_metadata.save()

        # Should not return error
        document_metadata.value = TEST_VALID_DATE
        document_metadata.full_clean()
        document_metadata.save()

        self.assertEqual(self.document.metadata_value_of.test, TEST_VALID_DATE)

    def test_parsing(self):
        self.metadata_type.parser = TEST_DATE_PARSER

        document_metadata = DocumentMetadata(
            document=self.document, metadata_type=self.metadata_type, value=TEST_INVALID_DATE
        )

        with self.assertRaises(ValidationError):
            # Should return error
            document_metadata.full_clean()
            document_metadata.save()

        # Should not return error
        document_metadata.value = TEST_VALID_DATE
        document_metadata.full_clean()
        document_metadata.save()

        self.assertEqual(self.document.metadata_value_of.test, TEST_PARSED_VALID_DATE)



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import EventListView, ObjectEventListView, VerbEventListView

urlpatterns = patterns(
    'events.views',
    url(r'^all/$', EventListView.as_view(), name='events_list'),
    url(
        r'^for/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/$',
        ObjectEventListView.as_view(), name='events_for_object'
    ),
    url(
        r'^by_verb/(?P<verb>[\w\-]+)/$', VerbEventListView.as_view(),
        name='events_by_verb'
    ),
)



from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from common.forms import DetailForm

from .models import (
    Document, DocumentType, DocumentPage, DocumentTypeFilename
)
from .literals import DEFAULT_ZIP_FILENAME, PAGE_RANGE_CHOICES
from .widgets import DocumentPagesCarouselWidget, DocumentPageImageWidget


# Document page forms
class DocumentPageForm(DetailForm):
    class Meta:
        fields = ()
        model = DocumentPage

    def __init__(self, *args, **kwargs):
        zoom = kwargs.pop('zoom', 100)
        rotation = kwargs.pop('rotation', 0)
        super(DocumentPageForm, self).__init__(*args, **kwargs)
        self.fields['page_image'].initial = self.instance
        self.fields['page_image'].widget.attrs.update({
            'zoom': zoom,
            'rotation': rotation
        })

    page_image = forms.CharField(
        label=_('Page image'), widget=DocumentPageImageWidget()
    )


# Document forms


class DocumentPreviewForm(forms.Form):
    def __init__(self, *args, **kwargs):
        document = kwargs.pop('document', None)
        super(DocumentPreviewForm, self).__init__(*args, **kwargs)
        self.fields['preview'].initial = document
        try:
            self.fields['preview'].label = _(
                'Document pages (%d)'
            ) % document.page_count
        except AttributeError:
            self.fields['preview'].label = _('Document pages (%d)') % 0

    preview = forms.CharField(widget=DocumentPagesCarouselWidget())


class DocumentForm(forms.ModelForm):
    """
    Form sub classes from DocumentForm used only when editing a document
    """
    class Meta:
        model = Document
        fields = ('label', 'description', 'language')

    def __init__(self, *args, **kwargs):
        document_type = kwargs.pop('document_type', None)

        super(DocumentForm, self).__init__(*args, **kwargs)

        # Is a document (documents app edit) and has been saved (sources
        # app upload)?
        if self.instance and self.instance.pk:
            document_type = self.instance.document_type

        filenames_qs = document_type.filenames.filter(enabled=True)
        if filenames_qs.count():
            self.fields[
                'document_type_available_filenames'
            ] = forms.ModelChoiceField(
                queryset=filenames_qs,
                required=False,
                label=_('Quick document rename')
            )

    def clean(self):
        if 'document_type_available_filenames' in self.cleaned_data:
            if self.cleaned_data['document_type_available_filenames']:
                self.cleaned_data['label'] = self.cleaned_data[
                    'document_type_available_filenames'
                ]

        return self.cleaned_data


class DocumentPropertiesForm(DetailForm):
    """
    Detail class form to display a document file based properties
    """
    class Meta:
        fields = ('document_type', 'description', 'language')
        model = Document


class DocumentTypeSelectForm(forms.Form):
    """
    Form to select the document type of a document to be created, used
    as form #1 in the document creation wizard
    """
    document_type = forms.ModelChoiceField(
        queryset=DocumentType.objects.all(), label=('Document type')
    )


class DocumentTypeFilenameForm_create(forms.ModelForm):
    """
    Model class form to create a new document type filename
    """
    class Meta:
        fields = ('filename',)
        model = DocumentTypeFilename


class DocumentDownloadForm(forms.Form):
    compressed = forms.BooleanField(
        label=_('Compress'), required=False,
        help_text=_(
            'Download the document in the original format or in a compressed '
            'manner. This option is selectable only when downloading one '
            'document, for multiple documents, the bundle will always be '
            'downloads as a compressed file.'
        )
    )
    zip_filename = forms.CharField(
        initial=DEFAULT_ZIP_FILENAME, label=_('Compressed filename'),
        required=False,
        help_text=_(
            'The filename of the compressed file that will contain the '
            'documents to be downloaded, if the previous option is selected.'
        )
    )

    def __init__(self, *args, **kwargs):
        self.document_versions = kwargs.pop('document_versions', None)
        super(DocumentDownloadForm, self).__init__(*args, **kwargs)
        if len(self.document_versions) > 1:
            self.fields['compressed'].initial = True
            self.fields['compressed'].widget.attrs.update({'disabled': True})


class PrintForm(forms.Form):
    page_group = forms.ChoiceField(
        widget=forms.RadioSelect, choices=PAGE_RANGE_CHOICES
    )
    page_range = forms.CharField(label=_('Page range'), required=False)



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .api_views import (
    APIRecentSearchListView, APIRecentSearchView, APISearchView
)
from .views import AdvancedSearchView, ResultsView, SearchView

urlpatterns = patterns(
    'dynamic_search.views',
    url(r'^$', SearchView.as_view(), name='search'),
    url(r'^advanced/$', AdvancedSearchView.as_view(), name='search_advanced'),
    url(r'^again/$', 'search_again', name='search_again'),
    url(r'^results/$', ResultsView.as_view(), name='results'),
)

api_urls = patterns(
    '',
    url(
        r'^recent_searches/$', APIRecentSearchListView.as_view(),
        name='recentsearch-list'
    ),
    url(
        r'^recent_searches/(?P<pk>[0-9]+)/$', APIRecentSearchView.as_view(),
        name='recentsearch-detail'
    ),
    url(r'^search/$', APISearchView.as_view(), name='search-view'),
)



from __future__ import unicode_literals

from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _

from common import (
    MayanAppConfig, menu_facet, menu_object, menu_secondary, menu_setup,
    menu_sidebar
)
from common.widgets import two_state_template
from documents.models import Document
from navigation import SourceColumn

from .handlers import launch_workflow
from .models import (
    Workflow, WorkflowInstance, WorkflowInstanceLogEntry, WorkflowState,
    WorkflowTransition
)
from .links import (
    link_document_workflow_instance_list, link_setup_workflow_document_types,
    link_setup_workflow_create, link_setup_workflow_delete,
    link_setup_workflow_edit, link_setup_workflow_list,
    link_setup_workflow_states, link_setup_workflow_state_create,
    link_setup_workflow_state_delete, link_setup_workflow_state_edit,
    link_setup_workflow_transitions, link_setup_workflow_transition_create,
    link_setup_workflow_transition_delete, link_setup_workflow_transition_edit,
    link_workflow_instance_detail, link_workflow_instance_transition
)


class DocumentStatesApp(MayanAppConfig):
    app_url = 'states'
    name = 'document_states'
    verbose_name = _('Document states')

    def ready(self):
        super(DocumentStatesApp, self).ready()

        SourceColumn(
            source=Workflow, label=_('Initial state'),
            func=lambda context: context['object'].get_initial_state() or _('None')
        )

        SourceColumn(
            source=WorkflowInstance, label=_('Current state'),
            attribute='get_current_state'
        )
        SourceColumn(
            source=WorkflowInstance, label=_('User'),
            func=lambda context: getattr(
                context['object'].get_last_log_entry(), 'user', _('None')
            )
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Last transition'),
            attribute='get_last_transition'
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Date and time'),
            func=lambda context: getattr(
                context['object'].get_last_log_entry(), 'datetime', _('None')
            )
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Completion'),
            func=lambda context: getattr(
                context['object'].get_current_state(), 'completion', _('None')
            )
        )

        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('Date and time'),
            attribute='datetime'
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('User'), attribute='user'
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('Transition'),
            attribute='transition'
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('Comment'),
            attribute='comment'
        )

        SourceColumn(
            source=WorkflowState, label=_('Is initial state?'),
            func=lambda context: two_state_template(context['object'].initial)
        )
        SourceColumn(
            source=WorkflowState, label=_('Completion'), attribute='completion'
        )

        SourceColumn(
            source=WorkflowTransition, label=_('Origin state'),
            attribute='origin_state'
        )
        SourceColumn(
            source=WorkflowTransition, label=_('Destination state'),
            attribute='destination_state'
        )

        menu_facet.bind_links(
            links=(link_document_workflow_instance_list,), sources=(Document,)
        )
        menu_object.bind_links(
            links=(
                link_setup_workflow_states, link_setup_workflow_transitions,
                link_setup_workflow_document_types, link_setup_workflow_edit,
                link_setup_workflow_delete
            ), sources=(Workflow,)
        )
        menu_object.bind_links(
            links=(
                link_setup_workflow_state_edit,
                link_setup_workflow_state_delete
            ), sources=(WorkflowState,)
        )
        menu_object.bind_links(
            links=(
                link_setup_workflow_transition_edit,
                link_setup_workflow_transition_delete
            ), sources=(WorkflowTransition,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_instance_detail,
                link_workflow_instance_transition
            ), sources=(WorkflowInstance,)
        )
        menu_secondary.bind_links(
            links=(link_setup_workflow_list, link_setup_workflow_create),
            sources=(
                Workflow, 'document_states:setup_workflow_create',
                'document_states:setup_workflow_list'
            )
        )
        menu_setup.bind_links(links=(link_setup_workflow_list,))
        menu_sidebar.bind_links(
            links=(
                link_setup_workflow_state_create,
                link_setup_workflow_transition_create
            ), sources=(Workflow,)
        )

        post_save.connect(
            launch_workflow, dispatch_uid='launch_workflow', sender=Document
        )



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .api_views import DocumentVersionOCRView
from .views import (
    DocumentAllSubmitView, DocumentSubmitView, DocumentSubmitManyView,
    DocumentTypeSettingsEditView, EntryListView
)

urlpatterns = patterns(
    'ocr.views',
    url(
        r'^(?P<document_id>\d+)/content/$', 'document_content',
        name='document_content'
    ),
    url(
        r'^document/(?P<pk>\d+)/submit/$', DocumentSubmitView.as_view(),
        name='document_submit'
    ),
    url(
        r'^document/all/submit/$', DocumentAllSubmitView.as_view(),
        name='document_submit_all'
    ),
    url(
        r'^document/multiple/submit/$', DocumentSubmitManyView.as_view(),
        name='document_submit_multiple'
    ),
    url(
        r'^document_type/(?P<pk>\d+)/ocr/settings/$',
        DocumentTypeSettingsEditView.as_view(),
        name='document_type_ocr_settings'
    ),

    url(r'^all/$', EntryListView.as_view(), name='entry_list'),
    url(r'^(?P<pk>\d+)/delete/$', 'entry_delete', name='entry_delete'),
    url(
        r'^multiple/delete/$', 'entry_delete_multiple',
        name='entry_delete_multiple'
    ),
    url(r'^(?P<pk>\d+)/re-queue/$', 'entry_re_queue', name='entry_re_queue'),
    url(
        r'^multiple/re-queue/$', 'entry_re_queue_multiple',
        name='entry_re_queue_multiple'
    ),
)

api_urls = patterns(
    '',
    url(
        r'^submit/$', DocumentVersionOCRView.as_view(),
        name='document-version-ocr-submit-view'
    ),
)



from .api import GPG
from .settings import setting_gpg_home, setting_gpg_path, setting_keyservers

gpg = GPG(
    binary_path=setting_gpg_path.value, home=setting_gpg_home.value,
    keyservers=setting_keyservers.value
)



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions import PermissionNamespace

namespace = PermissionNamespace(
    'document_signatures', _('Document signatures')
)

permission_document_verify = namespace.add_permission(
    name='document_verify', label=_('Verify document signatures')
)
permission_signature_delete = namespace.add_permission(
    name='signature_delete', label=_('Delete detached signatures')
)
permission_signature_download = namespace.add_permission(
    name='signature_download', label=_('Download detached signatures')
)
permission_signature_upload = namespace.add_permission(
    name='signature_upload', label=_('Upload detached signatures')
)



from __future__ import unicode_literals

from django.contrib import messages
from django.contrib.formtools.wizard.views import SessionWizardView
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _

from common.mixins import ViewPermissionCheckMixin
from documents.forms import DocumentTypeSelectForm
from documents.permissions import permission_document_create
from metadata.forms import MetadataFormSet

from .models import InteractiveSource


class DocumentCreateWizard(ViewPermissionCheckMixin, SessionWizardView):
    form_list = (DocumentTypeSelectForm, MetadataFormSet)
    template_name = 'appearance/generic_wizard.html'
    extra_context = {}
    view_permission = permission_document_create

    @staticmethod
    def _has_metadata_types(wizard):
        # Skip the 2nd step if document type has no associated metadata
        try:
            return wizard.get_cleaned_data_for_step('0')['document_type'].metadata.all().count()
        except TypeError:
            return False

    def dispatch(self, request, *args, **kwargs):
        if InteractiveSource.objects.filter(enabled=True).count() == 0:
            messages.error(
                request,
                _(
                    'No interactive document sources have been defined or '
                    'none have been enabled, create one before proceeding.'
                )
            )
            return HttpResponseRedirect(reverse('sources:setup_source_list'))
        return super(
            DocumentCreateWizard, self
        ).dispatch(request, *args, **kwargs)

    def __init__(self, *args, **kwargs):
        super(DocumentCreateWizard, self).__init__(*args, **kwargs)

        self.condition_dict = {'1': DocumentCreateWizard._has_metadata_types}

        self.step_titles = [
            _('Step 1 of 2: Select document type'),
            _('Step 2 of 2: Enter document metadata'),
        ]

    def get_form_initial(self, step):
        if step == '1':
            initial = []

            for document_type_metadata_type in self.get_cleaned_data_for_step('0')['document_type'].metadata.all():
                initial.append({
                    'document_type': self.get_cleaned_data_for_step('0')['document_type'],
                    'metadata_type': document_type_metadata_type.metadata_type,
                })

            return initial
        return self.initial_dict.get(step, {})

    def get_context_data(self, form, **kwargs):
        context = super(
            DocumentCreateWizard, self
        ).get_context_data(form=form, **kwargs)
        context.update({
            'step_title': self.step_titles[self.steps.step0],
            'submit_label': _('Next step'),
            'submit_icon': 'fa fa-arrow-right',
            'title': _('Document upload wizard'),
        })
        return context

    def done(self, *args, **kwargs):
        query_dict = {}
        try:
            query_dict['document_type_id'] = self.get_cleaned_data_for_step('0')['document_type'].pk
        except AttributeError:
            pass

        try:
            for identifier, metadata in enumerate(self.get_cleaned_data_for_step('1')):
                query_dict['metadata%s_id' % identifier] = metadata['id']
                query_dict['metadata%s_value' % identifier] = metadata['value']
        except TypeError:
            pass

        url = '?'.join(
            [
                reverse('sources:upload_interactive'),
                urlencode(query_dict, doseq=True)
            ]
        )
        return HttpResponseRedirect(url)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _


DELETE_STALE_UPLOADS_INTERVAL = 60 * 10  # 10 minutes
TIME_DELTA_UNIT_CHOICES = (
    ('days', _('Days')),
    ('hours', _('Hours')),
    ('minutes', _('Minutes')),
)
UPLOAD_EXPIRATION_INTERVAL = 60 * 60 * 24 * 7  # 7 days



from __future__ import unicode_literals

import time

from django.core.files import File
from django.test import TestCase, override_settings

from .literals import (
    TEST_DOCUMENT_TYPE, TEST_DOCUMENT_PATH, TEST_MULTI_PAGE_TIFF_PATH,
    TEST_OFFICE_DOCUMENT_PATH, TEST_SMALL_DOCUMENT_PATH
)
from ..models import DeletedDocument, Document, DocumentType


@override_settings(OCR_AUTO_OCR=False)
class DocumentTestCase(TestCase):
    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object), label='mayan_11_1.pdf'
            )

    def tearDown(self):
        self.document_type.delete()

    def test_document_creation(self):
        self.assertEqual(self.document_type.label, TEST_DOCUMENT_TYPE)

        self.assertEqual(self.document.exists(), True)
        self.assertEqual(self.document.size, 272213)

        self.assertEqual(self.document.file_mimetype, 'application/pdf')
        self.assertEqual(self.document.file_mime_encoding, 'binary')
        self.assertEqual(self.document.label, 'mayan_11_1.pdf')
        self.assertEqual(
            self.document.checksum,
            'c637ffab6b8bb026ed3784afdb07663fddc60099853fae2be93890852a69ecf3'
        )
        self.assertEqual(self.document.page_count, 47)

    def test_version_creation(self):
        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document.new_version(file_object=File(file_object))

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document.new_version(
                file_object=File(file_object), comment='test comment 1'
            )

        self.assertEqual(self.document.versions.count(), 3)

    def test_restoring_documents(self):
        self.assertEqual(Document.objects.count(), 1)

        # Trash the document
        self.document.delete()
        self.assertEqual(DeletedDocument.objects.count(), 1)
        self.assertEqual(Document.objects.count(), 0)

        # Restore the document
        self.document.restore()
        self.assertEqual(DeletedDocument.objects.count(), 0)
        self.assertEqual(Document.objects.count(), 1)

    def test_trashing_documents(self):
        self.assertEqual(Document.objects.count(), 1)

        # Trash the document
        self.document.delete()
        self.assertEqual(DeletedDocument.objects.count(), 1)
        self.assertEqual(Document.objects.count(), 0)

        # Delete the document
        self.document.delete()
        self.assertEqual(DeletedDocument.objects.count(), 0)
        self.assertEqual(Document.objects.count(), 0)


@override_settings(OCR_AUTO_OCR=False)
class OfficeDocumentTestCase(TestCase):
    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_OFFICE_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()

    def test_document_creation(self):
        self.assertEqual(self.document.file_mimetype, 'application/msword')
        self.assertEqual(self.document.file_mime_encoding, 'application/mswordbinary')
        self.assertEqual(
            self.document.checksum,
            '03a7e9071d2c6ae05a6588acd7dff1d890fac2772cf61abd470c9ffa6ef71f03'
        )
        self.assertEqual(self.document.page_count, 2)


@override_settings(OCR_AUTO_OCR=False)
class MultiPageTiffTestCase(TestCase):
    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_MULTI_PAGE_TIFF_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()

    def test_document_creation(self):
        self.assertEqual(self.document.file_mimetype, 'image/tiff')
        self.assertEqual(self.document.file_mime_encoding, 'binary')
        self.assertEqual(
            self.document.checksum,
            '40adaa9d658b65c70a7f002dfe084a8354bb77c0dfbf1993e31fb024a285fb1d'
        )
        self.assertEqual(self.document.page_count, 2)


@override_settings(OCR_AUTO_OCR=False)
class DocumentVersionTestCase(TestCase):
    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()

    def test_add_new_version(self):
        self.assertEqual(self.document.versions.count(), 1)

        with open(TEST_DOCUMENT_PATH) as file_object:
            self.document.new_version(
                file_object=File(file_object)
            )

        self.assertEqual(self.document.versions.count(), 2)

        self.assertEqual(
            self.document.checksum,
            'c637ffab6b8bb026ed3784afdb07663fddc60099853fae2be93890852a69ecf3'
        )

    def test_revert_version(self):
        self.assertEqual(self.document.versions.count(), 1)

        # Needed by MySQL as milliseconds value is not store in timestamp field
        time.sleep(1)

        with open(TEST_DOCUMENT_PATH) as file_object:
            self.document.new_version(
                file_object=File(file_object)
            )

        self.assertEqual(self.document.versions.count(), 2)

        self.document.versions.first().revert()

        self.assertEqual(self.document.versions.count(), 1)



from __future__ import unicode_literals

import logging
import uuid

from django.db import models
from django.utils.translation import ugettext_lazy as _

from django_gpg.runtime import gpg
from documents.models import DocumentVersion

from .managers import DocumentVersionSignatureManager
from .runtime import storage_backend

logger = logging.getLogger(__name__)


def upload_to(*args, **kwargs):
    return unicode(uuid.uuid4())


class DocumentVersionSignature(models.Model):
    """
    Model that describes a document version signature properties
    """
    document_version = models.ForeignKey(
        DocumentVersion, editable=False, verbose_name=_('Document version')
    )
    signature_file = models.FileField(
        blank=True, null=True, storage=storage_backend, upload_to=upload_to,
        verbose_name=_('Signature file')
    )
    has_embedded_signature = models.BooleanField(
        default=False, verbose_name=_('Has embedded signature')
    )

    objects = DocumentVersionSignatureManager()

    def check_for_embedded_signature(self):
        logger.debug('checking for embedded signature')

        with self.document_version.open(raw=True) as file_object:
            self.has_embedded_signature = gpg.has_embedded_signature(
                file_object
            )
            self.save()

    def delete_detached_signature_file(self):
        self.signature_file.storage.delete(self.signature_file.name)

    class Meta:
        verbose_name = _('Document version signature')
        verbose_name_plural = _('Document version signatures')



from __future__ import unicode_literals

import logging

from django.apps import apps
from django.db import models

from .settings import setting_recent_count

logger = logging.getLogger(__name__)


class RecentDocumentManager(models.Manager):
    def add_document_for_user(self, user, document):
        if user.is_authenticated():
            new_recent, created = self.model.objects.get_or_create(
                user=user, document=document
            )
            if not created:
                # document already in the recent list, just save to force
                # accessed date and time update
                new_recent.save()

            recent_to_delete = self.filter(user=user).values_list('pk', flat=True)[setting_recent_count.value:]
            self.filter(pk__in=list(recent_to_delete)).delete()

    def get_for_user(self, user):
        document_model = apps.get_model('documents', 'document')

        if user.is_authenticated():
            return document_model.objects.filter(
                recentdocument__user=user
            ).order_by('-recentdocument__datetime_accessed')
        else:
            return document_model.objects.none()


class DocumentTypeManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)


class DocumentManager(models.Manager):
    def get_queryset(self):
        return TrashCanQuerySet(
            self.model, using=self._db
        ).filter(in_trash=False)

    def invalidate_cache(self):
        for document in self.model.objects.all():
            document.invalidate_cache()


class PassthroughManager(models.Manager):
    pass


class TrashCanManager(models.Manager):
    def get_queryset(self):
        return super(
            TrashCanManager, self
        ).get_queryset().filter(in_trash=True)


class TrashCanQuerySet(models.QuerySet):
    def delete(self, to_trash=True):
        for instance in self:
            instance.delete(to_trash=to_trash)



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .api_views import (
    APIDocumentMetadataListView, APIDocumentMetadataView,
    APIDocumentTypeMetadataTypeOptionalListView,
    APIDocumentTypeMetadataTypeRequiredListView,
    APIDocumentTypeMetadataTypeRequiredView, APIMetadataTypeListView,
    APIMetadataTypeView
)
from .views import (
    DocumentMetadataListView, MetadataTypeCreateView, MetadataTypeDeleteView,
    MetadataTypeEditView, MetadataTypeListView,
    SetupDocumentTypeMetadataOptionalView,
    SetupDocumentTypeMetadataRequiredView
)

urlpatterns = patterns(
    'metadata.views',
    url(r'^(?P<document_id>\d+)/edit/$', 'metadata_edit', name='metadata_edit'),
    url(
        r'^(?P<pk>\d+)/view/$', DocumentMetadataListView.as_view(),
        name='metadata_view'
    ),
    url(
        r'^multiple/edit/$', 'metadata_multiple_edit',
        name='metadata_multiple_edit'
    ),
    url(r'^(?P<document_id>\d+)/add/$', 'metadata_add', name='metadata_add'),
    url(
        r'^multiple/add/$', 'metadata_multiple_add',
        name='metadata_multiple_add'
    ),
    url(
        r'^(?P<document_id>\d+)/remove/$', 'metadata_remove',
        name='metadata_remove'
    ),
    url(
        r'^multiple/remove/$', 'metadata_multiple_remove',
        name='metadata_multiple_remove'
    ),

    url(
        r'^setup/type/list/$', MetadataTypeListView.as_view(),
        name='setup_metadata_type_list'
    ),
    url(
        r'^setup/type/create/$', MetadataTypeCreateView.as_view(),
        name='setup_metadata_type_create'
    ),
    url(
        r'^setup/type/(?P<pk>\d+)/edit/$', MetadataTypeEditView.as_view(),
        name='setup_metadata_type_edit'
    ),
    url(
        r'^setup/type/(?P<pk>\d+)/delete/$',
        MetadataTypeDeleteView.as_view(), name='setup_metadata_type_delete'
    ),

    url(
        r'^setup/document/type/(?P<pk>\d+)/metadata/edit/$',
        SetupDocumentTypeMetadataOptionalView.as_view(),
        name='setup_document_type_metadata'
    ),
    url(
        r'^setup/document/type/(?P<pk>\d+)/metadata/edit/required/$',
        SetupDocumentTypeMetadataRequiredView.as_view(),
        name='setup_document_type_metadata_required'
    ),
)

api_urls = patterns(
    '',
    url(
        r'^metadatatypes/$', APIMetadataTypeListView.as_view(),
        name='metadatatype-list'
    ),
    url(
        r'^metadatatypes/(?P<pk>[0-9]+)/$', APIMetadataTypeView.as_view(),
        name='metadatatype-detail'
    ),
    url(
        r'^document/metadata/(?P<pk>[0-9]+)/$',
        APIDocumentMetadataView.as_view(), name='documentmetadata-detail'
    ),
    url(
        r'^document/(?P<document_pk>[0-9]+)/metadata/$',
        APIDocumentMetadataListView.as_view(), name='documentmetadata-list'
    ),
    url(
        r'^document_type/(?P<document_type_pk>[0-9]+)/metadatatypes/optional/$',
        APIDocumentTypeMetadataTypeOptionalListView.as_view(),
        name='documenttypemetadatatype-list'
    ),
    url(
        r'^document_type/(?P<document_type_pk>[0-9]+)/metadatatypes/required/$',
        APIDocumentTypeMetadataTypeRequiredListView.as_view(),
        name='documenttypemetadatatype-list'
    ),
    url(
        r'^document_type/(?P<document_type_pk>[0-9]+)/metadatatypes/(?P<metadata_type_pk>[0-9]+)/$',
        APIDocumentTypeMetadataTypeRequiredView.as_view(),
        name='documenttypemetadatatype-detail'
    ),
)



from __future__ import unicode_literals

from django.conf.urls import patterns, url

urlpatterns = patterns(
    'document_signatures.views',
    url(
        r'^verify/(?P<document_pk>\d+)/$', 'document_verify',
        name='document_verify'
    ),
    url(
        r'^upload/signature/(?P<document_pk>\d+)/$',
        'document_signature_upload', name='document_signature_upload'
    ),
    url(
        r'^download/signature/(?P<document_pk>\d+)/$',
        'document_signature_download', name='document_signature_download'
    ),
    url(
        r'^document/(?P<document_pk>\d+)/signature/delete/$',
        'document_signature_delete', name='document_signature_delete'
    ),
)



from __future__ import unicode_literals

from django.contrib.staticfiles.templatetags.staticfiles import static
from django.core.urlresolvers import reverse
from django.utils.html import strip_tags
from django.utils.http import urlencode
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from converter.literals import (
    DEFAULT_PAGE_NUMBER, DEFAULT_ROTATION, DEFAULT_ZOOM_LEVEL
)
from documents.settings import setting_preview_size, setting_thumbnail_size


def staging_file_thumbnail(staging_file, **kwargs):
    return staging_file_html_widget(
        staging_file, click_view='rest_api:stagingfolderfile-image-view', **kwargs
    )


def staging_file_html_widget(staging_file, click_view=None, page=DEFAULT_PAGE_NUMBER, zoom=DEFAULT_ZOOM_LEVEL, rotation=DEFAULT_ROTATION, gallery_name=None, fancybox_class='fancybox-staging', image_class='lazy-load', title=None, size=setting_thumbnail_size.value, nolazyload=False):
    result = []

    alt_text = _('Staging file page image')

    query_dict = {
        'page': page,
        'zoom': zoom,
        'rotation': rotation,
        'size': size,
    }

    if gallery_name:
        gallery_template = 'rel="%s"' % gallery_name
    else:
        gallery_template = ''

    query_string = urlencode(query_dict)

    preview_view = '%s?%s' % (
        reverse(
            'rest_api:stagingfolderfile-image-view',
            args=(
                staging_file.staging_folder.pk, staging_file.encoded_filename,
            )
        ), query_string
    )

    plain_template = []
    plain_template.append(
        '<img src="%s" alt="%s" />' % (preview_view, alt_text)
    )

    result.append(
        '<div class="tc" id="staging_file-%s-%d">' % (
            staging_file.filename, page if page else DEFAULT_PAGE_NUMBER
        )
    )

    if title:
        title_template = 'title="%s"' % strip_tags(title)
    else:
        title_template = ''

    if click_view:
        # TODO: fix this hack
        query_dict['size'] = setting_preview_size.value
        query_string = urlencode(query_dict)
        result.append(
            '<a %s class="%s" href="%s" %s>' % (
                gallery_template, fancybox_class,
                '%s?%s' % (
                    reverse(
                        click_view,
                        args=(
                            staging_file.staging_folder.pk,
                            staging_file.encoded_filename,
                        )
                    ),
                    query_string
                ), title_template
            )
        )

    if nolazyload:
        result.append(
            '<img style="border: 1px solid black;" src="%s" alt="%s" />' % (
                preview_view, alt_text
            )
        )
    else:
        result.append(
            '<img class="thin_border %s" data-src="%s" src="%s" alt="%s" />' % (
                image_class, preview_view,
                static('appearance/images/loading.png'), alt_text
            )
        )

    if click_view:
        result.append('</a>')
    result.append('</div>')

    return mark_safe(''.join(result))



import logging

from mayan.celery import app

from documents.models import DocumentType

from .models import DocumentMetadata, MetadataType

logger = logging.getLogger(__name__)


@app.task(ignore_result=True)
def task_remove_metadata_type(document_type_id, metadata_type_id):
    DocumentMetadata.objects.filter(
        document__document_type__id=document_type_id,
        metadata_type__id=metadata_type_id
    ).delete()


@app.task(ignore_result=True)
def task_add_required_metadata_type(document_type_id, metadata_type_id):
    metadata_type = MetadataType.objects.get(pk=metadata_type_id)

    for document in DocumentType.objects.get(pk=document_type_id).documents.all():
        document.metadata.create(metadata_type=metadata_type)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.html import mark_safe
from django.utils.translation import ugettext

from .models import IndexInstanceNode


def get_instance_link(index_instance_node, text=None, simple=False):
    """
    Return an HTML anchor to an index instance
    """

    if simple:
        # Just display the instance's value or overrided text, no
        # HTML anchor
        template = '%(value)s'
    else:
        template = '<a href="%(url)s">%(value)s</a>'

    return template % {
        'url': index_instance_node.get_absolute_url(),
        'value': text if text else (
            index_instance_node if index_instance_node.parent else index_instance_node.index_template_node.index
        )
    }


def get_breadcrumbs(index_instance_node, simple=False, single_link=False, include_count=False):
    """
    Return a joined string of HTML anchors to every index instance's
    parent from the root of the tree to the index instance
    """

    result = []
    if single_link:
        # Return the entire breadcrumb path as a single HTML anchor
        simple = True

    for instance in index_instance_node.get_ancestors():
        result.append(get_instance_link(instance, simple=simple))

    result.append(get_instance_link(index_instance_node, simple=simple))

    output = []

    if include_count:
        output.append('(%d)' % index_instance_node.documents.count())

    if single_link:
        # Return the entire breadcrumb path as a single HTML anchor
        output.insert(
            0, get_instance_link(
                index_instance_node=index_instance_node, text=(
                    ' / '.join(result)
                )
            )
        )
        return mark_safe(' '.join(output))
    else:
        output.insert(0, ' / '.join(result))
        return mark_safe(' '.join(output))


def index_instance_item_link(index_instance_item):
    if isinstance(index_instance_item, IndexInstanceNode):
        if index_instance_item.index_template_node.link_documents:
            icon_template = '<i class="fa fa-folder"></i>'
        else:
            icon_template = '<i class="fa fa-level-up fa-rotate-90"></i>'
    else:
        icon_template = ''

    return mark_safe(
        '%(icon_template)s&nbsp;<a href="%(url)s">%(text)s</a>' % {
            'url': index_instance_item.get_absolute_url(),
            'icon_template': icon_template,
            'text': index_instance_item
        }
    )


def node_level(node):
    """
    Render an indented tree like output for a specific node
    """

    return mark_safe(
        ''.join(
            [
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' * (
                    getattr(node, node._mptt_meta.level_attr) - 1
                ), '' if node.is_root_node() else '<i class="fa fa-level-up fa-rotate-90"></i> ',
                ugettext('Root') if node.is_root_node() else unicode(node)
            ]
        )
    )



from __future__ import unicode_literals

TEST_ADMIN_EMAIL = 'admin@admin.com'
TEST_ADMIN_PASSWORD = 'test_admin_password'
TEST_ADMIN_USERNAME = 'test_admin'
TEST_EMAIL_AUTHENTICATION_BACKEND = 'authentication.auth.email_auth_backend.EmailAuthBackend'



from __future__ import absolute_import, unicode_literals

import logging

from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import redirect, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from common.generics import SimpleView, SingleObjectListView
from permissions import Permission

from .api import Key
from .forms import KeySearchForm
from .permissions import (
    permission_key_delete, permission_key_receive, permission_key_view,
    permission_keyserver_query
)
from .runtime import gpg

logger = logging.getLogger(__name__)


def key_receive(request, key_id):
    Permission.check_permissions(request.user, (permission_key_receive,))

    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        try:
            gpg.receive_key(key_id=key_id)
        except Exception as exception:
            messages.error(
                request,
                _('Unable to import key: %(key_id)s; %(error)s') %
                {
                    'key_id': key_id,
                    'error': exception,
                }
            )
            return HttpResponseRedirect(previous)
        else:
            messages.success(
                request,
                _('Successfully received key: %(key_id)s') %
                {
                    'key_id': key_id,
                }
            )

            return redirect('django_gpg:key_public_list')

    return render_to_response('appearance/generic_confirm.html', {
        'message': _('Import key ID: %s?') % key_id,
        'previous': previous,
        'title': _('Import key'),
    }, context_instance=RequestContext(request))


class PublicKeyListView(SingleObjectListView):
    view_permission = permission_key_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'title': self.get_title()
        }

    def get_queryset(self):
        return Key.get_all(gpg)

    def get_title(self):
        return _('Public keys')


class PrivateKeyListView(PublicKeyListView):
    def get_title(self):
        return _('Private keys')

    def get_queryset(self):
        return Key.get_all(gpg, secret=True)


def key_delete(request, fingerprint, key_type):
    Permission.check_permissions(request.user, (permission_key_delete,))

    secret = key_type == 'sec'
    key = Key.get(gpg, fingerprint, secret=secret)

    post_action_redirect = redirect('django_gpg:key_public_list')
    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    next = request.POST.get('next', request.GET.get('next', post_action_redirect if post_action_redirect else request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        try:
            gpg.delete_key(key)
            messages.success(request, _('Key: %s, deleted successfully.') % fingerprint)
            return HttpResponseRedirect(next)
        except Exception as exception:
            messages.error(request, exception)
            return HttpResponseRedirect(previous)

    return render_to_response('appearance/generic_confirm.html', {
        'title': _('Delete key'),
        'delete_view': True,
        'message': _(
            'Delete key %s? If you delete a public key that is part of a '
            'public/private pair the private key will be deleted as well.'
        ) % key,
        'next': next,
        'previous': previous,
    }, context_instance=RequestContext(request))


class KeyQueryView(SimpleView):
    template_name = 'appearance/generic_form.html'
    view_permission = permission_keyserver_query

    def get_form(self):
        if ('term' in self.request.GET) and self.request.GET['term'].strip():
            term = self.request.GET['term']
            return KeySearchForm(initial={'term': term})
        else:
            return KeySearchForm()

    def get_extra_context(self):
        return {
            'form': self.get_form(),
            'form_action': reverse('django_gpg:key_query_results'),
            'submit_icon': 'fa fa-search',
            'submit_label': _('Search'),
            'submit_method': 'GET',
            'title': _('Query key server'),
        }


class KeyQueryResultView(SingleObjectListView):
    view_permission = permission_keyserver_query

    def get_extra_context(self):
        return {
            'hide_object': True,
            'title': _('Key query results'),
        }

    def get_queryset(self):
        term = self.request.GET.get('term')
        if term:
            return gpg.query(term)
        else:
            return ()



from __future__ import unicode_literals

from .tasks import task_delete_empty_index_nodes, task_index_document


def document_created_index_update(sender, **kwargs):
    task_index_document.apply_async(
        kwargs=dict(document_id=kwargs['instance'].pk)
    )


def document_index_delete(sender, **kwargs):
    task_delete_empty_index_nodes.apply_async()


def document_metadata_index_update(sender, **kwargs):
    task_index_document.apply_async(
        kwargs=dict(document_id=kwargs['instance'].document.pk)
    )


def document_metadata_index_post_delete(sender, **kwargs):
    task_index_document.apply_async(
        kwargs=dict(document_id=kwargs['instance'].document.pk)
    )



from __future__ import unicode_literals

import shutil

from django.test import TestCase

from ..api import GPG, Key
from ..settings import setting_gpg_path

TEST_GPG_HOME = '/tmp/test_gpg_home'
TEST_KEY_ID = '607138F1AECC5A5CA31CB7715F3F7F75D210724D'
TEST_KEYSERVERS = ['pool.sks-keyservers.net']
TEST_UIDS = 'Roberto Rosario'


class DjangoGPGTestCase(TestCase):
    def setUp(self):
        try:
            shutil.rmtree(TEST_GPG_HOME)
        except OSError:
            pass

        self.gpg = GPG(
            binary_path=setting_gpg_path.value, home=TEST_GPG_HOME,
            keyservers=TEST_KEYSERVERS
        )

    def test_main(self):
        # No private or public keys in the keyring
        self.assertEqual(Key.get_all(self.gpg, secret=True), [])
        self.assertEqual(Key.get_all(self.gpg), [])

        # Test querying the keyservers
        self.assertTrue(
            TEST_KEY_ID in [
                key_stub.key_id for key_stub in self.gpg.query(TEST_UIDS)
            ]
        )

        # Receive a public key from the keyserver
        self.gpg.receive_key(key_id=TEST_KEY_ID[-8:])

        # Check that the received key is indeed in the keyring
        self.assertTrue(
            TEST_KEY_ID[-16:] in [
                key_stub.key_id for key_stub in Key.get_all(self.gpg)
            ]
        )



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase, override_settings

from documents.models import Document, DocumentType
from documents.tests import (
    TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME, TEST_ADMIN_EMAIL,
    TEST_DOCUMENT_PATH, TEST_SMALL_DOCUMENT_PATH,
    TEST_DOCUMENT_DESCRIPTION, TEST_DOCUMENT_TYPE,
)
from sources.literals import SOURCE_CHOICE_WEB_FORM
from sources.models import WebFormSource


@override_settings(OCR_AUTO_OCR=False)
class UploadDocumentTestCase(TestCase):
    """
    Test creating documents
    """

    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        self.admin_user = User.objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )
        self.client = Client()

    def tearDown(self):
        self.document_type.delete()
        self.admin_user.delete()

    def test_upload_a_document(self):
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

        # Create new webform source
        self.client.post(
            reverse(
                'sources:setup_source_create', args=(SOURCE_CHOICE_WEB_FORM,)
            ), {'label': 'test', 'uncompress': 'n', 'enabled': True}
        )
        self.assertEqual(WebFormSource.objects.count(), 1)

        # Upload the test document
        with open(TEST_DOCUMENT_PATH) as file_descriptor:
            self.client.post(
                reverse('sources:upload_interactive'),
                {
                    'document-language': 'eng', 'source-file': file_descriptor,
                    'document_type_id': self.document_type.pk,
                    'label': 'mayan_11_1.pdf'
                }
            )
        self.assertEqual(Document.objects.count(), 1)

        self.document = Document.objects.all().first()
        self.assertEqual(self.document.exists(), True)
        self.assertEqual(self.document.size, 272213)

        self.assertEqual(self.document.file_mimetype, 'application/pdf')
        self.assertEqual(self.document.file_mime_encoding, 'binary')
        self.assertEqual(self.document.label, 'mayan_11_1.pdf')
        self.assertEqual(
            self.document.checksum,
            'c637ffab6b8bb026ed3784afdb07663fddc60099853fae2be93890852a69ecf3'
        )
        self.assertEqual(self.document.page_count, 47)

    def test_issue_25(self):
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

        # Create new webform source
        self.client.post(
            reverse(
                'sources:setup_source_create', args=(SOURCE_CHOICE_WEB_FORM,)
            ), {'label': 'test', 'uncompress': 'n', 'enabled': True}
        )
        self.assertEqual(WebFormSource.objects.count(), 1)

        # Upload the test document
        with open(TEST_SMALL_DOCUMENT_PATH) as file_descriptor:
            self.client.post(
                reverse('sources:upload_interactive'),
                {
                    'document-language': 'eng', 'source-file': file_descriptor,
                    'document_type_id': self.document_type.pk
                }
            )
        self.assertEqual(Document.objects.count(), 1)

        document = Document.objects.first()
        # Test for issue 25 during creation
        # ** description fields was removed from upload from **
        self.assertEqual(document.description, '')

        # Reset description
        document.description = TEST_DOCUMENT_DESCRIPTION
        document.save()
        self.assertEqual(document.description, TEST_DOCUMENT_DESCRIPTION)

        # Test for issue 25 during editing
        self.client.post(
            reverse('documents:document_edit', args=(document.pk,)),
            {
                'description': TEST_DOCUMENT_DESCRIPTION,
                'language': document.language, 'label': document.label
            }
        )
        # Fetch document again and test description
        document = Document.objects.first()
        self.assertEqual(document.description, TEST_DOCUMENT_DESCRIPTION)



from __future__ import unicode_literals

from rest_framework import serializers
from rest_framework.reverse import reverse


class APIVersionSerializer(serializers.Serializer):
    url = serializers.SerializerMethodField()
    version_string = serializers.CharField()

    def get_url(self, instance):
        return reverse(
            'rest_api:api-version-1', format=self.context['format'],
            request=self.context['request']
        )


class APIAppSerializer(serializers.Serializer):
    app_name = serializers.CharField()
    url = serializers.SerializerMethodField()
    version_string = serializers.CharField()

    def get_url(self, instance):
        return reverse(
            'rest_api:api-version-1-app', args=(instance.app.name,),
            format=self.context['format'], request=self.context['request']
        )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from documents.permissions import permission_document_view
from navigation import Link

from .permissions import (
    permission_folder_add_document, permission_folder_create,
    permission_folder_delete, permission_folder_edit, permission_folder_view,
    permission_folder_remove_document
)

link_document_folder_list = Link(
    permissions=(permission_document_view,), text=_('Folders'),
    view='folders:document_folder_list', args='object.pk'
)
link_folder_add_document = Link(
    permissions=(permission_folder_add_document,), text=_('Add to a folder'),
    view='folders:folder_add_document', args='object.pk'
)
link_folder_add_multiple_documents = Link(
    text=_('Add to folder'), view='folders:folder_add_multiple_documents'
)
link_folder_create = Link(
    permissions=(permission_folder_create,), text=_('Create folder'),
    view='folders:folder_create'
)
link_folder_delete = Link(
    permissions=(permission_folder_delete,), tags='dangerous',
    text=_('Delete'), view='folders:folder_delete', args='object.pk'
)
link_folder_document_multiple_remove = Link(
    permissions=(permission_folder_remove_document,),
    text=_('Remove from folder'),
    view='folders:folder_document_multiple_remove', args='object.pk'
)
link_folder_edit = Link(
    permissions=(permission_folder_edit,), text=_('Edit'),
    view='folders:folder_edit', args='object.pk'
)
link_folder_list = Link(
    icon='fa fa-folder', text=_('Folders'), view='folders:folder_list'
)
link_folder_view = Link(
    permissions=(permission_folder_view,), text=_('Documents'),
    view='folders:folder_view', args='object.pk'
)



from __future__ import unicode_literals, absolute_import

import logging

from permissions.models import StoredPermission

logger = logging.getLogger(__name__)


class ModelPermission(object):
    _registry = {}
    _proxies = {}
    _inheritances = {}

    @classmethod
    def register(cls, model, permissions):
        cls._registry.setdefault(model, [])
        for permission in permissions:
            cls._registry[model].append(permission)

    @classmethod
    def get_for_instance(cls, instance):
        try:
            permissions = cls._registry[type(instance)]
        except KeyError:
            try:
                permissions = cls._registry[cls._proxies[type(instance)]]
            except KeyError:
                permissions = ()

        pks = [permission.stored_permission.pk for permission in permissions]
        return StoredPermission.objects.filter(pk__in=pks)

    @classmethod
    def register_proxy(cls, source, model):
        cls._proxies[model] = source

    @classmethod
    def register_inheritance(cls, model, related):
        cls._inheritances[model] = related

    @classmethod
    def get_inheritance(cls, model):
        return cls._inheritances[model]



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from .models import DocumentType


def create_default_document_type(sender, **kwargs):
    DocumentType.objects.create(label=_('Default'))



from __future__ import unicode_literals

from django import forms
from django.forms.formsets import formset_factory
from django.utils.translation import string_concat, ugettext_lazy as _

from .classes import MetadataLookup
from .models import MetadataType


class MetadataForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super(MetadataForm, self).__init__(*args, **kwargs)

        # Set form fields initial values
        if 'initial' in kwargs:
            self.metadata_type = kwargs['initial'].pop('metadata_type', None)
            self.document_type = kwargs['initial'].pop('document_type', None)
            required_string = ''

            required = self.metadata_type.get_required_for(document_type=self.document_type)
            if required:
                self.fields['value'].required = True
                required_string = ' (%s)' % _('Required')
            else:
                self.fields['value'].required = False

            self.fields['name'].initial = '%s%s' % (
                (
                    self.metadata_type.label if self.metadata_type.label else self.metadata_type.name
                ),
                required_string
            )
            self.fields['id'].initial = self.metadata_type.pk

            if self.metadata_type.lookup:
                try:
                    self.fields['value'] = forms.ChoiceField(
                        label=self.fields['value'].label
                    )
                    choices = self.metadata_type.get_lookup_values()
                    choices = zip(choices, choices)
                    if not required:
                        choices.insert(0, ('', '------'))
                    self.fields['value'].choices = choices
                    self.fields['value'].required = required
                except Exception as exception:
                    self.fields['value'].initial = _(
                        'Lookup value error: %s'
                    ) % exception
                    self.fields['value'].widget = forms.TextInput(
                        attrs={'readonly': 'readonly'}
                    )

            if self.metadata_type.default:
                try:
                    self.fields[
                        'value'
                    ].initial = self.metadata_type.get_default_value()
                except Exception as exception:
                    self.fields['value'].initial = _(
                        'Default value error: %s'
                    ) % exception
                    self.fields['value'].widget = forms.TextInput(
                        attrs={'readonly': 'readonly'}
                    )

    def clean_value(self):
        return self.metadata_type.validate_value(document_type=self.document_type, value=self.cleaned_data['value'])

    id = forms.CharField(label=_('ID'), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Name'), required=False,
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )
    value = forms.CharField(label=_('Value'), required=False)
    update = forms.BooleanField(
        initial=True, label=_('Update'), required=False
    )

MetadataFormSet = formset_factory(MetadataForm, extra=0)


class AddMetadataForm(forms.Form):
    metadata_type = forms.ModelChoiceField(
        queryset=MetadataType.objects.all(), label=_('Metadata type')
    )

    def __init__(self, *args, **kwargs):
        document_type = kwargs.pop('document_type')
        super(AddMetadataForm, self).__init__(*args, **kwargs)
        self.fields['metadata_type'].queryset = document_type.metadata.all()


class MetadataTypeForm(forms.ModelForm):
    class Meta:
        fields = ('name', 'label', 'default', 'lookup', 'validation', 'parser')
        model = MetadataType

    def __init__(self, *args, **kwargs):
        super(MetadataTypeForm, self).__init__(*args, **kwargs)
        self.fields['lookup'].help_text = string_concat(
            self.fields['lookup'].help_text,
            _(' Available template context variables: '),
            MetadataLookup.get_as_help_text()
        )


class MetadataRemoveForm(MetadataForm):
    update = forms.BooleanField(
        initial=False, label=_('Remove'), required=False
    )

    def __init__(self, *args, **kwargs):
        super(MetadataRemoveForm, self).__init__(*args, **kwargs)
        self.fields.pop('value')


MetadataRemoveFormSet = formset_factory(MetadataRemoveForm, extra=0)



from __future__ import unicode_literals

import io
import logging

from django.utils.translation import ugettext_lazy as _

from acls import ModelPermission
from common import MayanAppConfig, menu_facet, menu_sidebar
from django_gpg.exceptions import GPGDecryptionError
from django_gpg.runtime import gpg
from documents.models import Document, DocumentVersion

from .links import (
    link_document_signature_delete, link_document_signature_download,
    link_document_signature_upload, link_document_verify
)
from .models import DocumentVersionSignature
from .permissions import (
    permission_document_verify, permission_signature_delete,
    permission_signature_download, permission_signature_upload
)

logger = logging.getLogger(__name__)


def document_pre_open_hook(descriptor, instance):
    if DocumentVersionSignature.objects.has_embedded_signature(document_version=instance):
        # If it has an embedded signature, decrypt
        try:
            result = gpg.decrypt_file(descriptor, close_descriptor=False)
            # gpg return a string, turn it into a file like object
        except GPGDecryptionError:
            # At least return the original raw content
            descriptor.seek(0)
            return descriptor
        else:
            descriptor.close()
            return io.BytesIO(result.data)
    else:
        return descriptor


def document_version_post_save_hook(instance):
    logger.debug('instance: %s', instance)

    try:
        document_signature = DocumentVersionSignature.objects.get(
            document_version=instance
        )
    except DocumentVersionSignature.DoesNotExist:
        document_signature = DocumentVersionSignature.objects.create(
            document_version=instance
        )
        document_signature.check_for_embedded_signature()


class DocumentSignaturesApp(MayanAppConfig):
    app_namespace = 'signatures'
    app_url = 'signatures'
    name = 'document_signatures'
    verbose_name = _('Document signatures')

    def ready(self):
        super(DocumentSignaturesApp, self).ready()

        DocumentVersion.register_post_save_hook(
            1, document_version_post_save_hook
        )
        DocumentVersion.register_pre_open_hook(1, document_pre_open_hook)

        ModelPermission.register(
            model=Document, permissions=(
                permission_document_verify, permission_signature_delete,
                permission_signature_download, permission_signature_upload,
            )
        )

        menu_facet.bind_links(
            links=(link_document_verify,), sources=(Document,)
        )
        menu_sidebar.bind_links(
            links=(
                link_document_signature_upload,
                link_document_signature_download,
                link_document_signature_delete
            ), sources=(
                'signatures:document_verify',
                'signatures:document_signature_upload',
                'signatures:document_signature_download',
                'signatures:document_signature_delete'
            )
        )



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from converter.models import Transformation

from .literals import SOURCE_UNCOMPRESS_CHOICE_ASK
from .models import POP3Email, IMAPEmail, WatchFolderSource, WebFormSource


def create_default_document_source(sender, **kwargs):
    WebFormSource.objects.create(
        label=_('Default'), uncompress=SOURCE_UNCOMPRESS_CHOICE_ASK
    )


def copy_transformations_to_version(sender, **kwargs):
    instance = kwargs['instance']

    # TODO: Fix this, source should be previous version
    # TODO: Fix this, shouldn't this be at the documents app
    Transformation.objects.copy(
        source=instance.document, targets=instance.pages.all()
    )


def initialize_periodic_tasks(**kwargs):
    for source in POP3Email.objects.filter(enabled=True):
        source.save()

    for source in IMAPEmail.objects.filter(enabled=True):
        source.save()

    for source in WatchFolderSource.objects.filter(enabled=True):
        source.save()



from __future__ import absolute_import, unicode_literals

import logging

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied
from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext

from permissions.models import StoredPermission

from .classes import ModelPermission

logger = logging.getLogger(__name__)


class AccessControlListManager(models.Manager):
    """
    Implement a 3 tier permission system, involving a permissions, an actor
    and an object
    """

    def get_inherited_permissions(self, role, obj):
        try:
            instance = obj.first()
        except AttributeError:
            instance = obj
        else:
            if not instance:
                return StoredPermission.objects.none()

        try:
            parent_accessor = ModelPermission.get_inheritance(type(instance))
        except KeyError:
            return StoredPermission.objects.none()
        else:
            parent_object = getattr(instance, parent_accessor)
            content_type = ContentType.objects.get_for_model(parent_object)
            try:
                return self.get(
                    role=role, content_type=content_type,
                    object_id=parent_object.pk
                ).permissions.all()
            except self.model.DoesNotExist:
                return StoredPermission.objects.none()

    def check_access(self, permissions, user, obj, related=None):
        if user.is_superuser or user.is_staff:
            return True

        try:
            stored_permissions = [
                permission.stored_permission for permission in permissions
            ]
        except TypeError:
            stored_permissions = [permissions.stored_permission]

        if related:
            obj = getattr(obj, related)

        user_roles = []
        for group in user.groups.all():
            for role in group.roles.all():
                if set(stored_permissions).intersection(set(self.get_inherited_permissions(role=role, obj=obj))):
                    return True

                user_roles.append(role)

        # TODO: possible .exists() optimization
        if not self.filter(content_type=ContentType.objects.get_for_model(obj), object_id=obj.pk, permissions__in=stored_permissions, role__in=user_roles):
            raise PermissionDenied(ugettext('Insufficient access.'))

    def filter_by_access(self, permission, user, queryset, related=None):
        if user.is_superuser or user.is_staff:
            return queryset

        user_roles = []
        for group in user.groups.all():
            for role in group.roles.all():
                user_roles.append(role)

        try:
            parent_accessor = ModelPermission.get_inheritance(queryset.model)
        except KeyError:
            parent_acl_query = Q()
        else:
            instance = queryset.first()
            if instance:
                parent_object = getattr(instance, parent_accessor)
                parent_content_type = ContentType.objects.get_for_model(
                    parent_object
                )
                parent_queryset = self.filter(
                    content_type=parent_content_type, role__in=user_roles,
                    permissions=permission.stored_permission
                )
                parent_acl_query = Q(
                    **{
                        '{}__pk__in'.format(
                            parent_accessor
                        ): parent_queryset.values_list('object_id', flat=True)
                    }
                )
            else:
                parent_acl_query = Q()

        # Directly granted access
        content_type = ContentType.objects.get_for_model(queryset.model)
        acl_query = Q(pk__in=self.filter(
            content_type=content_type, role__in=user_roles,
            permissions=permission.stored_permission
        ).values_list('object_id', flat=True))

        return queryset.filter(parent_acl_query | acl_query)



from __future__ import absolute_import, unicode_literals

from datetime import datetime
import logging

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.template.defaultfilters import force_escape
from django.utils.translation import ugettext_lazy as _

from acls.models import AccessControlList
from django_gpg.literals import SIGNATURE_STATE_NONE, SIGNATURE_STATES
from documents.models import Document
from filetransfers.api import serve_file
from permissions import Permission

from .forms import DetachedSignatureForm
from .models import DocumentVersionSignature
from .permissions import (
    permission_document_verify, permission_signature_upload,
    permission_signature_download, permission_signature_delete
)

logger = logging.getLogger(__name__)


def document_verify(request, document_pk):
    document = get_object_or_404(Document, pk=document_pk)

    try:
        Permission.check_permissions(
            request.user, (permission_document_verify,)
        )
    except PermissionDenied:
        AccessControlList.objects.check_access(
            permission_document_verify, request.user, document
        )

    document.add_as_recent_document_for_user(request.user)

    try:
        signature = DocumentVersionSignature.objects.verify_signature(
            document.latest_version
        )
    except AttributeError:
        signature_state = SIGNATURE_STATES.get(SIGNATURE_STATE_NONE)
        signature = None
    else:
        signature_state = SIGNATURE_STATES.get(
            getattr(signature, 'status', None)
        )

    paragraphs = [_('Signature status: %s') % signature_state['text']]

    try:
        if DocumentVersionSignature.objects.has_embedded_signature(document.latest_version):
            signature_type = _('Embedded')
        else:
            signature_type = _('Detached')
    except ValueError:
        signature_type = _('None')

    if signature:
        paragraphs.extend(
            [
                _('Signature ID: %s') % signature.signature_id,
                _('Signature type: %s') % signature_type,
                _('Key ID: %s') % signature.key_id,
                _('Timestamp: %s') % datetime.fromtimestamp(
                    int(signature.sig_timestamp)
                ),
                _('Signee: %s') % force_escape(getattr(signature, 'username', '')),
            ]
        )

    return render_to_response('appearance/generic_template.html', {
        'document': document,
        'object': document,
        'paragraphs': paragraphs,
        'title': _('Signature properties for document: %s') % document,
    }, context_instance=RequestContext(request))


def document_signature_upload(request, document_pk):
    document = get_object_or_404(Document, pk=document_pk)

    try:
        Permission.check_permissions(
            request.user, (permission_signature_upload,)
        )
    except PermissionDenied:
        AccessControlList.objects.check_access(
            permission_signature_upload, request.user, document
        )

    document.add_as_recent_document_for_user(request.user)

    post_action_redirect = None
    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    next = request.POST.get('next', request.GET.get('next', post_action_redirect if post_action_redirect else request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        form = DetachedSignatureForm(request.POST, request.FILES)
        if form.is_valid():
            try:
                DocumentVersionSignature.objects.add_detached_signature(
                    document.latest_version, request.FILES['file']
                )
                messages.success(
                    request, _('Detached signature uploaded successfully.')
                )
                return HttpResponseRedirect(next)
            except Exception as exception:
                messages.error(request, exception)
                return HttpResponseRedirect(previous)
    else:
        form = DetachedSignatureForm()

    return render_to_response('appearance/generic_form.html', {
        'form': form,
        'next': next,
        'object': document,
        'previous': previous,
        'title': _('Upload detached signature for document: %s') % document,
    }, context_instance=RequestContext(request))


def document_signature_download(request, document_pk):
    document = get_object_or_404(Document, pk=document_pk)

    try:
        Permission.check_permissions(
            request.user, (permission_signature_download,)
        )
    except PermissionDenied:
        AccessControlList.objects.check_access(
            permission_signature_download, request.user, document
        )

    try:
        if DocumentVersionSignature.objects.has_detached_signature(document.latest_version):
            signature = DocumentVersionSignature.objects.detached_signature(
                document.latest_version
            )
            return serve_file(
                request,
                signature,
                save_as='"%s.sig"' % document.filename,
                content_type='application/octet-stream'
            )
    except Exception as exception:
        messages.error(request, exception)
        return HttpResponseRedirect(request.META['HTTP_REFERER'])

    return HttpResponseRedirect(request.META['HTTP_REFERER'])


def document_signature_delete(request, document_pk):
    document = get_object_or_404(Document, pk=document_pk)

    try:
        Permission.check_permissions(
            request.user, (permission_signature_delete,)
        )
    except PermissionDenied:
        AccessControlList.objects.check_access(
            permission_signature_delete, request.user, document
        )

    document.add_as_recent_document_for_user(request.user)

    post_action_redirect = None
    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    next = request.POST.get('next', request.GET.get('next', post_action_redirect if post_action_redirect else request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        try:
            DocumentVersionSignature.objects.clear_detached_signature(
                document.latest_version
            )
            messages.success(
                request, _('Detached signature deleted successfully.')
            )
            return HttpResponseRedirect(next)
        except Exception as exception:
            messages.error(
                request, _(
                    'Error while deleting the detached signature; %s'
                ) % exception
            )
            return HttpResponseRedirect(previous)

    return render_to_response('appearance/generic_confirm.html', {
        'delete_view': True,
        'next': next,
        'object': document,
        'previous': previous,
        'title': _(
            'Delete the detached signature from document: %s?'
        ) % document,
    }, context_instance=RequestContext(request))



from __future__ import unicode_literals

from datetime import timedelta
import logging

from django.utils.timezone import now

from mayan.celery import app

from .literals import UPLOAD_EXPIRATION_INTERVAL
from .models import SharedUploadedFile

logger = logging.getLogger(__name__)


@app.task(ignore_result=True)
def task_delete_stale_uploads():
    logger.info('Executing')

    for expired_upload in SharedUploadedFile.objects.filter(datetime__lt=now() - timedelta(seconds=UPLOAD_EXPIRATION_INTERVAL)):
        expired_upload.delete()

    logger.info('Finshed')



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.core.files import File
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase, override_settings

from .literals import (
    TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME, TEST_ADMIN_EMAIL,
    TEST_SMALL_DOCUMENT_PATH, TEST_DOCUMENT_TYPE
)
from ..models import DeletedDocument, Document, DocumentType


@override_settings(OCR_AUTO_OCR=False)
class DocumentsViewsFunctionalTestCase(TestCase):
    """
    Functional tests to make sure all the moving parts after creating a
    document from the frontend are working correctly
    """

    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        self.admin_user = User.objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )
        self.client = Client()
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object), label='mayan_11_1.pdf'
            )

    def tearDown(self):
        self.document_type.delete()
        self.admin_user.delete()

    def test_restoring_documents(self):
        self.assertEqual(Document.objects.count(), 1)

        # Trash the document
        self.client.post(
            reverse('documents:document_trash', args=(self.document.pk,))
        )
        self.assertEqual(DeletedDocument.objects.count(), 1)
        self.assertEqual(Document.objects.count(), 0)

        # Restore the document
        self.client.post(
            reverse('documents:document_restore', args=(self.document.pk,))
        )
        self.assertEqual(DeletedDocument.objects.count(), 0)
        self.assertEqual(Document.objects.count(), 1)

    def test_trashing_documents(self):
        self.assertEqual(Document.objects.count(), 1)

        # Trash the document
        self.client.post(
            reverse('documents:document_trash', args=(self.document.pk,))
        )
        self.assertEqual(DeletedDocument.objects.count(), 1)
        self.assertEqual(Document.objects.count(), 0)

        # Delete the document
        self.client.post(
            reverse('documents:document_delete', args=(self.document.pk,))
        )
        self.assertEqual(DeletedDocument.objects.count(), 0)
        self.assertEqual(Document.objects.count(), 0)

    def test_document_view(self):
        response = self.client.get(reverse('documents:document_list'))
        self.assertContains(response, 'Total: 1', status_code=200)

        # test document simple view
        response = self.client.get(
            reverse('documents:document_properties', args=(self.document.pk,))
        )
        self.assertContains(
            response, 'roperties for document', status_code=200
        )

    def test_document_type_views(self):
        # Check that there are no document types
        response = self.client.get(reverse('documents:document_type_list'))
        self.assertContains(response, 'Total: 1', status_code=200)

        # Create a document type
        response = self.client.post(
            reverse('documents:document_type_create'),
            {'name': 'test document type 2'}, follow=True
        )
        #TODO: FIX
        # self.assertContains(response, 'successfully', status_code=200)

        # Check that there are two document types
        response = self.client.get(reverse('documents:document_type_list'))
        #TODO: FIX self.assertContains(response, 'Total: 2', status_code=200)

        self.assertEqual(self.document_type.label, TEST_DOCUMENT_TYPE)

        # Edit the document type
        response = self.client.post(
            reverse(
                'documents:document_type_edit', args=(self.document_type.pk,)
            ), data={'name': TEST_DOCUMENT_TYPE + 'partial'}, follow=True
        )
        #TODO: FIX
        # self.assertContains(
        #    response, 'Document type edited successfully', status_code=200
        #)

        # Reload document type model data
        #self.document_type = DocumentType.objects.get(
        #    pk=self.document_type.pk
        #)
        #TODO: FIX#
        #self.assertEqual(
        #    self.document_type.name, TEST_DOCUMENT_TYPE + 'partial'
        #)

        # Delete the document type
        #response = self.client.post(
        #    reverse(
        #        'documents:document_type_delete', args=(self.document_type.pk,)
        #    ), follow=True
        #)
        #TODO: FIX#
        # self.assertContains(
        #    response, 'Document type: {0} deleted successfully'.format(
        #        self.document_type.name
        #    ), status_code=200
        #)

        # Check that there are no document types
        #response = self.client.get(reverse('documents:document_type_list'))
        #TODO: FIX self.assertEqual(response.status_code, 200)
        #TODO: FIX
        #self.assertContains(
        #    response, 'ocument types (0)', status_code=200
        #)



"""
Django settings for Mayan EDMS project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""
from __future__ import unicode_literals

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import sys

from django.utils.translation import ugettext_lazy as _, ugettext

_file_path = os.path.abspath(os.path.dirname(__file__)).split('/')

BASE_DIR = '/'.join(_file_path[0:-2])

MEDIA_ROOT = os.path.join(BASE_DIR, 'mayan', 'media')

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'secret_key_missing'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition

INSTALLED_APPS = (
    # Placed at the top so it can override any template
    'appearance.apps.AppearanceApp',
    # 3rd party
    'suit',
    # Django
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.comments',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',
    # 3rd party
    'actstream',
    'autoadmin',
    'colorful',
    'compressor',
    'corsheaders',
    'djcelery',
    'filetransfers',
    'mptt',
    'pure_pagination',
    'rest_framework',
    'rest_framework.authtoken',
    'solo',
    'widget_tweaks',
    # Base generic
    'acls.apps.ACLsApp',
    'authentication.apps.AuthenticationApp',
    'common.apps.CommonApp',
    'converter.apps.ConverterApp',
    'django_gpg.apps.DjangoGPGApp',
    'dynamic_search.apps.DynamicSearchApp',
    'lock_manager.apps.LockManagerApp',
    'mimetype.apps.MIMETypesApp',
    'navigation.apps.NavigationApp',
    'permissions.apps.PermissionsApp',
    'smart_settings.apps.SmartSettingsApp',
    'user_management.apps.UserManagementApp',
    # Mayan EDMS
    'checkouts.apps.CheckoutsApp',
    'document_comments.apps.DocumentCommentsApp',
    'document_indexing.apps.DocumentIndexingApp',
    'document_signatures.apps.DocumentSignaturesApp',
    'document_states.apps.DocumentStatesApp',
    'documents.apps.DocumentsApp',
    'events.apps.EventsApp',
    'folders.apps.FoldersApp',
    'installation.apps.InstallationApp',
    'linking.apps.LinkingApp',
    'mailer.apps.MailerApp',
    'metadata.apps.MetadataApp',
    'mirroring.apps.MirroringApp',
    'ocr.apps.OCRApp',
    'rest_api.apps.RESTAPIApp',
    'sources.apps.SourcesApp',
    'statistics.apps.StatisticsApp',
    'storage.apps.StorageApp',
    'tags.apps.TagsApp',
    # Placed after rest_api to allow template overriding
    'rest_framework_swagger',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'common.middleware.timezone.TimezoneMiddleware',
    'common.middleware.strip_spaces_widdleware.SpacelessMiddleware',
    'authentication.middleware.login_required_middleware.LoginRequiredMiddleware',
    'common.middleware.ajax_redirect.AjaxRedirect',
)

ROOT_URLCONF = 'mayan.urls'

WSGI_APPLICATION = 'mayan.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(MEDIA_ROOT, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'


# ------------ Custom settings section ----------

TEMPLATE_DEBUG = True
PROJECT_TITLE = 'Mayan EDMS'
PROJECT_NAME = 'mayan'
PROJECT_WEBSITE = 'http://www.mayan-edms.com'

LANGUAGES = (
    ('ar', _('Arabic')),
    ('bg', _('Bulgarian')),
    ('bs', _('Bosnian (Bosnia and Herzegovina)')),
    ('da', _('Danish')),
    ('de', _('German (Germany)')),
    ('en', _('English')),
    ('es', _('Spanish')),
    ('fa', _('Persian')),
    ('fr', _('French')),
    ('hu', _('Hungarian')),
    ('hr', _('Croatian')),
    ('id', _('Indonesian')),
    ('it', _('Italian')),
    ('nl', _('Dutch (Nethherlands)')),
    ('pl', _('Polish')),
    ('pt', _('Portuguese')),
    ('pt-br', _('Portuguese (Brazil)')),
    ('ro', _('Romanian (Romania)')),
    ('ru', _('Russian')),
    ('sl', _('Slovenian')),
    ('tr', _('Turkish')),
    ('vi', _('Vietnamese (Viet Nam)')),
    ('zh-cn', _('Chinese (China)')),
)

SITE_ID = 1

sys.path.append(os.path.join(BASE_DIR, 'mayan', 'apps'))

STATIC_ROOT = os.path.join(MEDIA_ROOT, 'static')

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader'
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder',
)

# --------- Django compressor -------------
COMPRESS_CSS_FILTERS = (
    'compressor.filters.css_default.CssAbsoluteFilter',
    'compressor.filters.cssmin.CSSMinFilter'
)
COMPRESS_ENABLED = False
COMPRESS_PARSER = 'compressor.parser.HtmlParser'
# --------- Django -------------------
LOGIN_URL = 'authentication:login_view'
LOGIN_REDIRECT_URL = 'common:home'
INTERNAL_IPS = ('127.0.0.1',)
# -------- LoginRequiredMiddleware ----------
LOGIN_EXEMPT_URLS = (
    r'^favicon\.ico$',
    r'^about\.html$',
    r'^legal/',  # allow the entire /legal/* subsection
    r'^%s-static/' % PROJECT_NAME,

    r'^accounts/register/$',
    r'^accounts/register/complete/$',
    r'^accounts/register/closed/$',

    r'^accounts/activate/complete/',
    r'^accounts/activate/(?P<activation_key>\w+)/$',

    r'^authentication/password/reset/$',
    r'^authentication/password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
    r'^authentication/password/reset/complete/$',
    r'^authentication/password/reset/done/$',

    r'^api/',
)
# ---------- Django REST framework -----------
REST_FRAMEWORK = {
    'PAGINATE_BY': 10,
    'PAGINATE_BY_PARAM': 'page_size',
    'MAX_PAGINATE_BY': 100,
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.TokenAuthentication',
        'rest_framework.authentication.SessionAuthentication',
    )
}
# --------- Pagination --------
PAGINATION_SETTINGS = {
    'PAGE_RANGE_DISPLAYED': 8,
    'MARGIN_PAGES_DISPLAYED': 2,
}
# ----------- Celery ----------
CELERY_ACCEPT_CONTENT = ('json',)
CELERY_ALWAYS_EAGER = True
CELERY_CREATE_MISSING_QUEUES = False
CELERY_DISABLE_RATE_LIMITS = True
CELERY_EAGER_PROPAGATES_EXCEPTIONS = True
CELERY_ENABLE_UTC = True
CELERY_QUEUES = []
CELERY_RESULT_SERIALIZER = 'json'
CELERY_ROUTES = {}
CELERY_TASK_SERIALIZER = 'json'
CELERY_TIMEZONE = 'UTC'
CELERYBEAT_SCHEDULER = 'djcelery.schedulers.DatabaseScheduler'
TEST_RUNNER = 'djcelery.contrib.test_runner.CeleryTestSuiteRunner'
# ------------ CORS ------------
CORS_ORIGIN_ALLOW_ALL = True
# ------ Django REST Swagger -----
SWAGGER_SETTINGS = {
    'api_version': '1',
    'info': {
          'title': _('Mayan EDMS API Documentation'),
          'description': _('Free Open Source Document Management System.'),
          'contact': 'roberto.rosario@mayan-edms.com',
          'license': 'Apache 2.0',
          'licenseUrl': 'http://www.apache.org/licenses/LICENSE-2.0.html'
    }

}
# ------ Timezone --------
TIMEZONE_COOKIE_NAME = 'django_timezone'
TIMEZONE_SESSION_KEY = 'django_timezone'



from __future__ import unicode_literals

from django.conf import settings
from django.conf.urls import patterns, url

urlpatterns = patterns(
    'authentication.views',
    url(r'^login/$', 'login_view', name='login_view'),
    url(
        r'^password/change/done/$', 'password_change_done',
        name='password_change_done'
    ),
    url(
        r'^password/change/$', 'password_change_view',
        name='password_change_view'
    ),
)

urlpatterns += patterns(
    '',
    url(
        r'^logout/$', 'django.contrib.auth.views.logout',
        {'next_page': settings.LOGIN_REDIRECT_URL}, name='logout_view'
    ),
    url(
        r'^password/reset/$', 'django.contrib.auth.views.password_reset',
        {
            'email_template_name': 'appearance/password_reset_email.html',
            'template_name': 'appearance/password_reset_form.html',
            'post_reset_redirect': '/password/reset/done'
        }, name='password_reset_view'
    ),
    url(
        r'^password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        'django.contrib.auth.views.password_reset_confirm', {
            'template_name': 'appearance/password_reset_confirm.html',
            'post_reset_redirect': '/password/reset/complete/'
        }, name='password_reset_confirm_view'
    ),
    url(
        r'^password/reset/complete/$',
        'django.contrib.auth.views.password_reset_complete', {
            'template_name': 'appearance/password_reset_complete.html'
        }, name='password_reset_complete_view'),
    url(
        r'^password/reset/done/$',
        'django.contrib.auth.views.password_reset_done', {
            'template_name': 'appearance/password_reset_done.html'
        }, name='password_reset_done_view'),
)



from __future__ import unicode_literals

from django.contrib.auth.models import Group, User

from rest_framework import generics

from rest_api.filters import MayanObjectPermissionsFilter
from rest_api.permissions import MayanPermission

from .permissions import (
    permission_group_create, permission_group_delete, permission_group_edit,
    permission_group_view, permission_user_create, permission_user_delete,
    permission_user_edit, permission_user_view
)
from .serializers import GroupSerializer, UserSerializer


class APIGroupListView(generics.ListCreateAPIView):
    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {'GET': (permission_group_view,)}
    mayan_view_permissions = {'POST': (permission_group_create,)}
    permission_classes = (MayanPermission,)
    queryset = Group.objects.all()
    serializer_class = GroupSerializer

    def get(self, *args, **kwargs):
        """
        Returns a list of all the groups.
        """

        return super(APIGroupListView, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        """
        Create a new group.
        """

        return super(APIGroupListView, self).post(*args, **kwargs)


class APIGroupView(generics.RetrieveUpdateDestroyAPIView):
    mayan_object_permissions = {
        'GET': (permission_group_view,),
        'PUT': (permission_group_edit,),
        'PATCH': (permission_group_edit,),
        'DELETE': (permission_group_delete,)
    }
    permission_classes = (MayanPermission,)
    queryset = Group.objects.all()
    serializer_class = GroupSerializer

    def delete(self, *args, **kwargs):
        """
        Delete the selected group.
        """

        return super(APIGroupView, self).delete(*args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Return the details of the selected group.
        """

        return super(APIGroupView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """
        Partially edit the selected group.
        """

        return super(APIGroupView, self).patch(*args, **kwargs)

    def put(self, *args, **kwargs):
        """
        Edit the selected group.
        """

        return super(APIGroupView, self).put(*args, **kwargs)


class APIUserListView(generics.ListCreateAPIView):
    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {'GET': (permission_user_view,)}
    mayan_view_permissions = {'POST': (permission_user_create,)}
    permission_classes = (MayanPermission,)
    queryset = User.objects.all()
    serializer_class = UserSerializer

    def get(self, *args, **kwargs):
        """
        Returns a list of all the users.
        """
        return super(APIUserListView, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        """
        Create a new user.
        """

        return super(APIUserListView, self).post(*args, **kwargs)


class APIUserView(generics.RetrieveUpdateDestroyAPIView):
    mayan_object_permissions = {
        'GET': (permission_user_view,),
        'PUT': (permission_user_edit,),
        'PATCH': (permission_user_edit,),
        'DELETE': (permission_user_delete,)
    }
    permission_classes = (MayanPermission,)
    queryset = User.objects.all()
    serializer_class = UserSerializer

    def delete(self, *args, **kwargs):
        """
        Delete the selected user.
        """

        return super(APIUserView, self).delete(*args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Return the details of the selected user.
        """

        return super(APIUserView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """
        Partially edit the selected user.
        """

        return super(APIUserView, self).patch(*args, **kwargs)

    def put(self, *args, **kwargs):
        """
        Edit the selected user.
        """

        return super(APIUserView, self).put(*args, **kwargs)


class APICurrentUserView(generics.RetrieveUpdateDestroyAPIView):
    serializer_class = UserSerializer

    def get_object(self):
        return self.request.user

    def delete(self, *args, **kwargs):
        """
        Delete the current user.
        """

        return super(APICurrentUserView, self).delete(*args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Return the details of the current user.
        """

        return super(APICurrentUserView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """
        Partially edit the current user.
        """

        return super(APICurrentUserView, self).patch(*args, **kwargs)

    def put(self, *args, **kwargs):
        """
        Edit the current user.
        """

        return super(APICurrentUserView, self).put(*args, **kwargs)



from __future__ import absolute_import, unicode_literals

from ..base import *  # NOQA


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(name)s %(process)d %(thread)d %(message)s'
        },
        'intermediate': {
            'format': '%(name)s <%(process)d> [%(levelname)s] "%(funcName)s() %(message)s"'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'WARNING',
            'class': 'logging.StreamHandler',
            'formatter': 'intermediate'
        }
    },
    'loggers': {
        'acls': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'converter': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'django_gpg': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'document_indexing': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'document_signatures': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'documents': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'dynamic_search': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'folders': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'lock_manager': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'ocr': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'permissions': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'sources': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'tags': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
    }
}


TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader'
)



from __future__ import unicode_literals

from rest_framework import generics
from rest_framework.exceptions import ParseError

from rest_api.filters import MayanObjectPermissionsFilter

from .classes import SearchModel
from .filters import RecentSearchUserFilter
from .models import RecentSearch
from .serializers import RecentSearchSerializer


class APIRecentSearchListView(generics.ListAPIView):
    """
    Returns a list of all the recent searches for the logged user.
    """

    filter_backends = (RecentSearchUserFilter,)
    queryset = RecentSearch.objects.all()
    serializer_class = RecentSearchSerializer


class APIRecentSearchView(generics.RetrieveAPIView):
    """
    Returns the selected recent search details.
    """

    filter_backends = (RecentSearchUserFilter,)
    queryset = RecentSearch.objects.all()
    serializer_class = RecentSearchSerializer


class APISearchView(generics.ListAPIView):
    """
    Perform a search operaton
    q -- Term that will be used for the search.
    """

    filter_backends = (MayanObjectPermissionsFilter,)

    def get_queryset(self):
        search_class = self.get_search_class()
        if search_class.permission:
            self.mayan_object_permissions = {'GET': (search_class.permission,)}

        try:
            queryset, ids, timedelta = search_class.search(
                query_string=self.request.GET, user=self.request.user
            )
        except Exception as exception:
            raise ParseError(unicode(exception))

        return queryset

    def get_search_class(self):
        return SearchModel.get('documents.Document')

    def get_serializer_class(self):
        return self.get_search_class().serializer



from __future__ import unicode_literals

from django.utils.html import escape
from django.utils.safestring import mark_safe


def widget_inline_tags(document):
    """
    A tag widget that displays the tags for the given document
    """
    tags_template = []

    for tag in document.attached_tags().all():
        tags_template.append(widget_single_tag(tag))

    return mark_safe(''.join(tags_template))


def widget_single_tag(tag):
    return mark_safe(
        '''
            <span class="label label-tag" style="background: {}">{}</span>
        '''.format(tag.color, escape(tag.label).replace(' ', '&nbsp;'))
    )



from __future__ import unicode_literals

import logging
import urlparse

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _

from common.generics import SimpleView, SingleObjectListView

from .classes import SearchModel
from .forms import SearchForm, AdvancedSearchForm
from .settings import setting_limit, setting_show_object_type

logger = logging.getLogger(__name__)


class ResultsView(SingleObjectListView):
    def get_extra_context(self):
        context = {
            'hide_links': True,
            'search_results_limit': setting_limit.value,
            'title': _('Search results'),
        }

        if setting_show_object_type.value:
            context.update({
                'extra_columns': (
                    {
                        'name': _('Type'),
                        'attribute': lambda x: x._meta.verbose_name[0].upper() + x._meta.verbose_name[1:]
                    },
                )
            })

        return context

    def get_queryset(self):
        document_search = SearchModel.get('documents.Document')

        if self.request.GET:
            # Only do search if there is user input, otherwise just render
            # the template with the extra_context

            queryset, ids, timedelta = document_search.search(self.request.GET, self.request.user)

            return queryset


class SearchView(SimpleView):
    template_name = 'appearance/generic_form.html'
    title = _('Search')

    def get_form(self):
        if ('q' in self.request.GET) and self.request.GET['q'].strip():
            query_string = self.request.GET['q']
            return SearchForm(initial={'q': query_string})
        else:
            return SearchForm()

    def get_extra_context(self):
        return {
            'form': self.get_form(),
            'form_action': reverse('search:results'),
            'submit_icon': 'fa fa-search',
            'submit_label': _('Search'),
            'submit_method': 'GET',
            'title': self.title,
        }


class AdvancedSearchView(SearchView):
    title = _('Advanced search')

    def get_form(self):
        document_search = SearchModel.get('documents.Document')

        return AdvancedSearchForm(
            data=self.request.GET, search_model=document_search
        )


def search_again(request):
    query = urlparse.urlparse(
        request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))
    ).query
    return HttpResponseRedirect(
        '{}?{}'.format(reverse('search:search_advanced'), query)
    )



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse, reverse_lazy
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _, ungettext

from acls.models import AccessControlList
from common.generics import (
    AssignRemoveView, SingleObjectCreateView, SingleObjectDeleteView,
    SingleObjectEditView, SingleObjectListView
)
from documents.models import Document, DocumentType
from documents.permissions import (
    permission_document_type_edit
)
from permissions import Permission

from .api import save_metadata_list
from .forms import (
    AddMetadataForm, MetadataFormSet, MetadataRemoveFormSet, MetadataTypeForm
)
from .models import DocumentMetadata, MetadataType
from .permissions import (
    permission_metadata_document_add, permission_metadata_document_edit,
    permission_metadata_document_remove, permission_metadata_document_view,
    permission_metadata_type_create, permission_metadata_type_delete,
    permission_metadata_type_edit, permission_metadata_type_view
)


def metadata_edit(request, document_id=None, document_id_list=None):
    if document_id:
        document_id_list = unicode(document_id)

    documents = Document.objects.select_related('metadata').filter(
        pk__in=document_id_list.split(',')
    )

    try:
        Permission.check_permissions(
            request.user, (permission_metadata_document_edit,)
        )
    except PermissionDenied:
        documents = AccessControlList.objects.filter_by_access(
            permission_metadata_document_edit, request.user, documents
        )

    if not documents:
        if document_id:
            raise Http404
        else:
            messages.error(request, _('Must provide at least one document.'))
            return HttpResponseRedirect(
                request.META.get(
                    'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
                )
            )

    if len(set([document.document_type.pk for document in documents])) > 1:
        messages.error(request, _('Only select documents of the same type.'))
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    if set(documents.values_list('metadata__value', flat=True)) == set([None]):
        message = ungettext(
            'The selected document doesn\'t have any metadata.',
            'The selected documents don\'t have any metadata.',
            len(documents)
        )
        messages.warning(request, message)
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    post_action_redirect = reverse('documents:document_list_recent')

    next = request.POST.get(
        'next', request.GET.get(
            'next', request.META.get('HTTP_REFERER', post_action_redirect)
        )
    )

    metadata = {}
    initial = []

    for document in documents:
        document.add_as_recent_document_for_user(request.user)

        for item in document.metadata.all():
            value = item.value
            if item.metadata_type in metadata:
                if value not in metadata[item.metadata_type]:
                    metadata[item.metadata_type].append(value)
            else:
                metadata[item.metadata_type] = [value] if value else []

    for key, value in metadata.items():
        initial.append({
            'document_type': document.document_type,
            'metadata_type': key,
            'value': ', '.join(value) if value else '',
        })

    formset = MetadataFormSet(initial=initial)
    if request.method == 'POST':
        formset = MetadataFormSet(data=request.POST, initial=initial)
        if formset.is_valid():
            for document in documents:

                errors = []
                for form in formset.forms:
                    if form.cleaned_data['update']:
                        try:
                            save_metadata_list([form.cleaned_data], document)
                        except Exception as exception:
                            errors.append(exception)

                if errors:
                    for error in errors:
                        if settings.DEBUG:
                            raise
                        else:
                            messages.error(
                                request, _(
                                    'Error editing metadata for document: '
                                    '%(document)s; %(exception)s.'
                                ) % {
                                    'document': document,
                                    'exception': ', '.join(exception.messages)
                                }
                            )
                else:
                    messages.success(
                        request,
                        _(
                            'Metadata for document %s edited successfully.'
                        ) % document
                    )

            return HttpResponseRedirect(next)

    context = {
        'form_display_mode_table': True,
        'form': formset,
        'next': next,
    }

    if len(documents) == 1:
        context['object'] = documents[0]

    context['title'] = ungettext(
        'Edit document metadata',
        'Edit documents metadata',
        len(documents)
    )

    return render_to_response(
        'appearance/generic_form.html', context,
        context_instance=RequestContext(request)
    )


def metadata_multiple_edit(request):
    return metadata_edit(
        request, document_id_list=request.GET.get('id_list', '')
    )


def metadata_add(request, document_id=None, document_id_list=None):
    if document_id:
        documents = [get_object_or_404(Document, pk=document_id)]
    elif document_id_list:
        documents = [
            get_object_or_404(
                Document.objects.select_related('document_type'),
                pk=document_id
            ) for document_id in document_id_list.split(',')
        ]
        if len(set([document.document_type.pk for document in documents])) > 1:
            messages.error(
                request, _('Only select documents of the same type.')
            )
            return HttpResponseRedirect(
                request.META.get(
                    'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
                )
            )

    try:
        Permission.check_permissions(
            request.user, (permission_metadata_document_add,)
        )
    except PermissionDenied:
        documents = AccessControlList.objects.filter_by_access(
            permission_metadata_document_add, request.user, documents
        )

    if not documents:
        messages.error(request, _('Must provide at least one document.'))
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    for document in documents:
        document.add_as_recent_document_for_user(request.user)

    post_action_redirect = reverse('documents:document_list_recent')

    next = request.POST.get(
        'next',
        request.GET.get(
            'next', request.META.get('HTTP_REFERER', post_action_redirect)
        )
    )

    if request.method == 'POST':
        form = AddMetadataForm(
            data=request.POST, document_type=document.document_type
        )
        if form.is_valid():
            metadata_type = form.cleaned_data['metadata_type']
            for document in documents:
                try:
                    document_metadata, created = DocumentMetadata.objects.get_or_create(
                        document=document,
                        metadata_type=metadata_type.metadata_type,
                        defaults={'value': ''}
                    )
                except Exception as exception:
                    if getattr(settings, 'DEBUG', False):
                        raise
                    else:
                        messages.error(
                            request,
                            _(
                                'Error adding metadata type '
                                '"%(metadata_type)s" to document: '
                                '%(document)s; %(exception)s'
                            ) % {
                                'metadata_type': metadata_type,
                                'document': document,
                                'exception': ', '.join(
                                    getattr(exception, 'messages', exception)
                                )
                            }
                        )
                else:
                    if created:
                        messages.success(
                            request,
                            _(
                                'Metadata type: %(metadata_type)s '
                                'successfully added to document %(document)s.'
                            ) % {
                                'metadata_type': metadata_type,
                                'document': document
                            }
                        )
                    else:
                        messages.warning(
                            request, _(
                                'Metadata type: %(metadata_type)s already '
                                'present in document %(document)s.'
                            ) % {
                                'metadata_type': metadata_type,
                                'document': document
                            }
                        )

            if len(documents) == 1:
                return HttpResponseRedirect('%s?%s' % (
                    reverse('metadata:metadata_edit', args=(document.pk,)),
                    urlencode({'next': next}))
                )
            elif len(documents) > 1:
                return HttpResponseRedirect('%s?%s' % (
                    reverse('metadata:metadata_multiple_edit'),
                    urlencode({'id_list': document_id_list, 'next': next}))
                )

    else:
        form = AddMetadataForm(document_type=document.document_type)

    context = {
        'form': form,
        'next': next,
    }

    if len(documents) == 1:
        context['object'] = documents[0]

    context['title'] = ungettext(
        'Add metadata types to document',
        'Add metadata types to documents',
        len(documents)
    )

    return render_to_response('appearance/generic_form.html', context,
                              context_instance=RequestContext(request))


def metadata_multiple_add(request):
    return metadata_add(
        request, document_id_list=request.GET.get('id_list', [])
    )


def metadata_remove(request, document_id=None, document_id_list=None):
    if document_id:
        document_id_list = unicode(document_id)

    documents = Document.objects.select_related('metadata').filter(
        pk__in=document_id_list.split(',')
    )

    try:
        Permission.check_permissions(
            request.user, (permission_metadata_document_remove,)
        )
    except PermissionDenied:
        documents = AccessControlList.objects.filter_by_access(
            permission_metadata_document_remove, request.user, documents
        )

    if not documents:
        if document_id:
            raise Http404
        else:
            messages.error(request, _('Must provide at least one document.'))
            return HttpResponseRedirect(
                request.META.get(
                    'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
                )
            )

    if len(set([document.document_type.pk for document in documents])) > 1:
        messages.error(request, _('Only select documents of the same type.'))
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    if set(documents.values_list('metadata__value', flat=True)) == set([None]):
        message = ungettext(
            'The selected document doesn\'t have any metadata.',
            'The selected documents doesn\'t have any metadata.',
            len(documents)
        )
        messages.warning(request, message)
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    post_action_redirect = reverse('documents:document_list_recent')

    next = request.POST.get(
        'next', request.GET.get(
            'next', request.META.get('HTTP_REFERER', post_action_redirect)
        )
    )

    metadata = {}
    for document in documents:
        document.add_as_recent_document_for_user(request.user)

        for item in document.metadata.all():
            value = item.value
            if item.metadata_type in metadata:
                if value not in metadata[item.metadata_type]:
                    metadata[item.metadata_type].append(value)
            else:
                metadata[item.metadata_type] = [value] if value else ''

    initial = []
    for key, value in metadata.items():
        initial.append({
            'metadata_type': key,
            'value': ', '.join(value)
        })

    formset = MetadataRemoveFormSet(initial=initial)
    if request.method == 'POST':
        formset = MetadataRemoveFormSet(request.POST)
        if formset.is_valid():
            for document in documents:

                for form in formset.forms:
                    if form.cleaned_data['update']:
                        metadata_type = get_object_or_404(
                            MetadataType, pk=form.cleaned_data['id']
                        )
                        try:
                            document_metadata = DocumentMetadata.objects.get(
                                document=document, metadata_type=metadata_type
                            )
                            document_metadata.delete()
                            messages.success(
                                request,
                                _(
                                    'Successfully remove metadata type "%(metadata_type)s" from document: %(document)s.'
                                ) % {
                                    'metadata_type': metadata_type,
                                    'document': document
                                }
                            )
                        except Exception as exception:
                            messages.error(
                                request,
                                _(
                                    'Error removing metadata type "%(metadata_type)s" from document: %(document)s; %(exception)s'
                                ) % {
                                    'metadata_type': metadata_type,
                                    'document': document,
                                    'exception': ', '.join(exception.messages)
                                }
                            )

            return HttpResponseRedirect(next)

    context = {
        'form_display_mode_table': True,
        'form': formset,
        'next': next,
    }

    if len(documents) == 1:
        context['object'] = documents[0]

    context['title'] = ungettext(
        'Remove metadata types from the document',
        'Remove metadata types from the documents',
        len(documents)
    )

    return render_to_response('appearance/generic_form.html', context,
                              context_instance=RequestContext(request))


def metadata_multiple_remove(request):
    return metadata_remove(
        request, document_id_list=request.GET.get('id_list', [])
    )


class DocumentMetadataListView(SingleObjectListView):
    def dispatch(self, request, *args, **kwargs):
        try:
            Permission.check_permissions(
                self.request.user, (permission_metadata_document_view,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_metadata_document_view, self.request.user,
                self.get_document()
            )

        return super(DocumentMetadataListView, self).dispatch(
            request, *args, **kwargs
        )

    def get_document(self):
        return get_object_or_404(Document, pk=self.kwargs['pk'])

    def get_extra_context(self):
        document = self.get_document()
        return {
            'hide_link': True,
            'object': document,
            'title': _('Metadata for document: %s') % document,
        }

    def get_queryset(self):
        return self.get_document().metadata.all()


# Setup views
class MetadataTypeCreateView(SingleObjectCreateView):
    extra_context = {'title': _('Create metadata type')}
    form_class = MetadataTypeForm
    model = MetadataType
    post_action_redirect = reverse_lazy('metadata:setup_metadata_type_list')
    view_permission = permission_metadata_type_create


class MetadataTypeDeleteView(SingleObjectDeleteView):
    model = MetadataType
    post_action_redirect = reverse_lazy('metadata:setup_metadata_type_list')
    view_permission = permission_metadata_type_delete

    def get_extra_context(self):
        return {
            'delete_view': True,
            'object': self.get_object(),
            'title': _('Delete the metadata type: %s?') % self.get_object(),
        }


class MetadataTypeEditView(SingleObjectEditView):
    form_class = MetadataTypeForm
    model = MetadataType
    post_action_redirect = reverse_lazy('metadata:setup_metadata_type_list')
    view_permission = permission_metadata_type_edit

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Edit metadata type: %s') % self.get_object(),
        }


class MetadataTypeListView(SingleObjectListView):
    view_permission = permission_metadata_type_view

    def get_queryset(self):
        return MetadataType.objects.all()

    def get_extra_context(self):
        return {
            'extra_columns': (
                {
                    'name': _('Internal name'),
                    'attribute': 'name',
                },
            ),
            'hide_link': True,
            'title': _('Metadata types'),
        }


class SetupDocumentTypeMetadataOptionalView(AssignRemoveView):
    decode_content_type = True
    view_permission = permission_document_type_edit
    left_list_title = _('Available metadata types')
    right_list_title = _('Metadata types assigned')

    def add(self, item):
        self.get_object().metadata.create(metadata_type=item, required=False)

    def get_object(self):
        return get_object_or_404(DocumentType, pk=self.kwargs['pk'])

    def left_list(self):
        return AssignRemoveView.generate_choices(
            set(MetadataType.objects.all()) - set(
                MetadataType.objects.filter(
                    id__in=self.get_object().metadata.values_list(
                        'metadata_type', flat=True
                    )
                )
            )
        )

    def right_list(self):
        return AssignRemoveView.generate_choices(
            self.get_object().metadata.filter(required=False)
        )

    def remove(self, item):
        item.delete()

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _(
                'Optional metadata types for document type: %s'
            ) % self.get_object(),
        }


class SetupDocumentTypeMetadataRequiredView(SetupDocumentTypeMetadataOptionalView):
    def add(self, item):
        self.get_object().metadata.create(metadata_type=item, required=True)

    def right_list(self):
        return AssignRemoveView.generate_choices(
            self.get_object().metadata.filter(required=True)
        )

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _(
                'Required metadata types for document type: %s'
            ) % self.get_object(),
        }



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied
from django.shortcuts import get_object_or_404

from rest_framework import generics, status
from rest_framework.response import Response
from rest_framework.settings import api_settings

from acls.models import AccessControlList
from documents.models import DocumentVersion
from permissions import Permission
from rest_api.permissions import MayanPermission

from .permissions import permission_ocr_document
from .serializers import DocumentVersionOCRSerializer


class DocumentVersionOCRView(generics.GenericAPIView):
    serializer_class = DocumentVersionOCRSerializer

    permission_classes = (MayanPermission,)

    def post(self, request, *args, **kwargs):
        """
        Submit document version for OCR.
        """

        serializer = self.get_serializer(
            data=request.DATA, files=request.FILES
        )

        if serializer.is_valid():
            document_version = get_object_or_404(
                DocumentVersion, pk=serializer.data['document_version_id']
            )

            try:
                Permission.check_permissions(
                    request.user, (permission_ocr_document,)
                )
            except PermissionDenied:
                AccessControlList.objects.check_access(
                    permission_ocr_document, request.user,
                    document_version.document
                )

            document_version.submit_for_ocr()

            headers = self.get_success_headers(serializer.data)
            return Response(serializer.data, status=status.HTTP_202_ACCEPTED,
                            headers=headers)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def get_success_headers(self, data):
        try:
            return {'Location': data[api_settings.URL_FIELD_NAME]}
        except (TypeError, KeyError):
            return {}



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.core.files.base import File
from django.test import TestCase

from authentication.tests.test_views import (
    TEST_ADMIN_EMAIL, TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME
)
from documents.models import DocumentType
from documents.tests import TEST_DOCUMENT_PATH, TEST_DOCUMENT_TYPE

from ..models import Folder


class FolderTestCase(TestCase):
    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

        self.user = User.objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )

    def tearDown(self):
        self.document.delete()
        self.document_type.delete()
        self.user.delete()

    def test_creation_of_folder(self):
        folder = Folder.objects.create(label='test', user=self.user)

        self.assertEqual(Folder.objects.all().count(), 1)
        self.assertEqual(list(Folder.objects.all()), [folder])
        folder.delete()

    def test_addition_of_documents(self):
        folder = Folder.objects.create(label='test', user=self.user)
        folder.documents.add(self.document)

        self.assertEqual(folder.documents.count(), 1)
        self.assertEqual(list(folder.documents.all()), [self.document])
        folder.delete()

    def test_addition_and_deletion_of_documents(self):
        folder = Folder.objects.create(label='test', user=self.user)
        folder.documents.add(self.document)

        self.assertEqual(folder.documents.count(), 1)
        self.assertEqual(list(folder.documents.all()), [self.document])

        folder.documents.remove(self.document)

        self.assertEqual(folder.documents.count(), 0)
        self.assertEqual(list(folder.documents.all()), [])

        folder.delete()



from __future__ import absolute_import, unicode_literals

import logging

from django import forms
from django.core.exceptions import PermissionDenied
from django.utils.translation import ugettext_lazy as _

from acls.models import AccessControlList
from permissions import Permission

from .models import Folder
from .permissions import permission_folder_view

logger = logging.getLogger(__name__)


class FolderListForm(forms.Form):
    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        logger.debug('user: %s', user)
        super(FolderListForm, self).__init__(*args, **kwargs)

        queryset = Folder.objects.all()
        try:
            Permission.check_permissions(user, (permission_folder_view,))
        except PermissionDenied:
            queryset = AccessControlList.objects.filter_by_access(
                permission_folder_view, user, queryset
            )

        self.fields['folder'] = forms.ModelChoiceField(
            queryset=queryset,
            label=_('Folder'))



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import (
    DocumentSmartLinkListView, ResolvedSmartLinkView,
    SetupSmartLinkDocumentTypesView, SmartLinkConditionListView,
    SmartLinkCreateView, SmartLinkDeleteView, SmartLinkEditView,
    SmartLinkListView
)

urlpatterns = patterns(
    'linking.views',
    url(
        r'^document/(?P<pk>\d+)/list/$', DocumentSmartLinkListView.as_view(),
        name='smart_link_instances_for_document'
    ),
    url(
        r'^document/(?P<document_pk>\d+)/(?P<smart_link_pk>\d+)/$',
        ResolvedSmartLinkView.as_view(), name='smart_link_instance_view'
    ),

    url(
        r'^setup/list/$', SmartLinkListView.as_view(), name='smart_link_list'
    ),
    url(
        r'^setup/create/$', SmartLinkCreateView.as_view(),
        name='smart_link_create'
    ),
    url(
        r'^setup/(?P<pk>\d+)/delete/$',
        SmartLinkDeleteView.as_view(), name='smart_link_delete'
    ),
    url(
        r'^setup/(?P<pk>\d+)/edit/$', SmartLinkEditView.as_view(),
        name='smart_link_edit'
    ),
    url(
        r'^setup/(?P<pk>\d+)/document_types/$',
        SetupSmartLinkDocumentTypesView.as_view(),
        name='smart_link_document_types'
    ),

    url(
        r'^setup/(?P<pk>\d+)/condition/list/$',
        SmartLinkConditionListView.as_view(), name='smart_link_condition_list'
    ),
    url(
        r'^setup/(?P<smart_link_pk>\d+)/condition/create/$',
        'smart_link_condition_create', name='smart_link_condition_create'
    ),
    url(
        r'^setup/smart_link/condition/(?P<smart_link_condition_pk>\d+)/edit/$',
        'smart_link_condition_edit', name='smart_link_condition_edit'
    ),
    url(
        r'^setup/smart_link/condition/(?P<smart_link_condition_pk>\d+)/delete/$',
        'smart_link_condition_delete', name='smart_link_condition_delete'
    ),
)



from __future__ import unicode_literals

import datetime
import time

from django.contrib.auth import get_user_model
from django.core.files import File
from django.test import TestCase, override_settings
from django.utils.timezone import now

from authentication.tests.literals import (
    TEST_ADMIN_USERNAME, TEST_ADMIN_EMAIL, TEST_ADMIN_PASSWORD
)
from documents.models import DocumentType
from documents.tests.literals import (
    TEST_DOCUMENT_TYPE, TEST_DOCUMENT_PATH, TEST_SMALL_DOCUMENT_PATH
)

from ..exceptions import (
    DocumentAlreadyCheckedOut, DocumentNotCheckedOut,
    NewDocumentVersionNotAllowed
)
from ..models import DocumentCheckout


@override_settings(OCR_AUTO_OCR=False)
class DocumentCheckoutTestCase(TestCase):
    def setUp(self):
        self.admin_user = get_user_model().objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()

    def test_document_checkout(self):
        expiration_datetime = now() + datetime.timedelta(days=1)

        DocumentCheckout.objects.checkout_document(
            document=self.document, expiration_datetime=expiration_datetime,
            user=self.admin_user, block_new_version=True
        )

        self.assertTrue(self.document.is_checked_out())
        self.assertTrue(
            DocumentCheckout.objects.is_document_checked_out(
                document=self.document
            )
        )

    def test_version_creation_blocking(self):
        expiration_datetime = now() + datetime.timedelta(days=1)

        DocumentCheckout.objects.checkout_document(
            document=self.document, expiration_datetime=expiration_datetime,
            user=self.admin_user, block_new_version=True
        )

        with self.assertRaises(NewDocumentVersionNotAllowed):
            with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
                self.document.new_version(file_object=File(file_object))

    def test_checkin_in(self):
        expiration_datetime = now() + datetime.timedelta(days=1)

        DocumentCheckout.objects.checkout_document(
            document=self.document, expiration_datetime=expiration_datetime,
            user=self.admin_user, block_new_version=True
        )

        self.document.check_in()

        self.assertFalse(self.document.is_checked_out())
        self.assertFalse(
            DocumentCheckout.objects.is_document_checked_out(
                document=self.document
            )
        )

    def test_double_checkout(self):
        expiration_datetime = now() + datetime.timedelta(days=1)

        DocumentCheckout.objects.checkout_document(
            document=self.document, expiration_datetime=expiration_datetime,
            user=self.admin_user, block_new_version=True
        )

        with self.assertRaises(DocumentAlreadyCheckedOut):
            DocumentCheckout.objects.checkout_document(
                document=self.document,
                expiration_datetime=expiration_datetime, user=self.admin_user,
                block_new_version=True
            )

    def test_checkin_without_checkout(self):
        with self.assertRaises(DocumentNotCheckedOut):
            self.document.check_in()

    def test_auto_checkin(self):
        expiration_datetime = now() + datetime.timedelta(seconds=1)

        DocumentCheckout.objects.checkout_document(
            document=self.document, expiration_datetime=expiration_datetime,
            user=self.admin_user, block_new_version=True
        )

        time.sleep(2)

        DocumentCheckout.objects.check_in_expired_check_outs()

        self.assertFalse(self.document.is_checked_out())



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from navigation import Link

from .models import DocumentVersionSignature
from .permissions import (
    permission_document_verify, permission_signature_delete,
    permission_signature_download, permission_signature_upload,
)


def can_upload_detached_signature(context):
    return not DocumentVersionSignature.objects.has_detached_signature(
        context['object'].latest_version
    ) and not DocumentVersionSignature.objects.has_embedded_signature(
        context['object'].latest_version
    )


def can_delete_detached_signature(context):
    return DocumentVersionSignature.objects.has_detached_signature(
        context['object'].latest_version
    )


link_document_signature_delete = Link(
    condition=can_delete_detached_signature,
    permissions=(permission_signature_delete,), tags='dangerous',
    text=_('Delete signature'), view='signatures:document_signature_delete',
    args='object.pk'
)
link_document_signature_download = Link(
    condition=can_delete_detached_signature, text=_('Download signature'),
    view='signatures:document_signature_download', args='object.pk',
    permissions=(permission_signature_download,)
)
link_document_signature_upload = Link(
    condition=can_upload_detached_signature,
    permissions=(permission_signature_upload,), text=_('Upload signature'),
    view='signatures:document_signature_upload', args='object.pk'
)
link_document_verify = Link(
    permissions=(permission_document_verify,), text=_('Signatures'),
    view='signatures:document_verify', args='object.pk'
)



from __future__ import unicode_literals

from django import forms

from .models import DocumentCheckout
from .widgets import SplitTimeDeltaWidget


class DocumentCheckoutForm(forms.ModelForm):
    class Meta:
        model = DocumentCheckout
        exclude = ('document', 'checkout_datetime', 'user')
        widgets = {
            'expiration_datetime': SplitTimeDeltaWidget()
        }



from __future__ import unicode_literals

DEFAULT_ZOOM_LEVEL = 100
DEFAULT_ROTATION = 0
DEFAULT_PAGE_NUMBER = 1
DEFAULT_FILE_FORMAT = 'jpeg'
DEFAULT_FILE_FORMAT_MIMETYPE = 'image/jpeg'

DIMENSION_SEPARATOR = 'x'



from __future__ import unicode_literals

import logging

from PIL import Image
import pytesseract

from ..classes import OCRBackendBase
from ..exceptions import OCRError
from ..settings import setting_tesseract_path

logger = logging.getLogger(__name__)


class Tesseract(OCRBackendBase):
    def execute(self, *args, **kwargs):
        """
        Execute the command line binary of tesseract
        """
        super(Tesseract, self).execute(*args, **kwargs)

        # TODO: pass tesseract binary path to the pytesseract
        image = Image.open(self.converter.get_page())
        try:
            result = pytesseract.image_to_string(
                image=image, lang=self.language
            )
            # If tesseract gives an error with a language parameter
            # re-run it with no language parameter
        except Exception as exception:
            error_message = 'Exception calling pytesseract with language option: {}; {}'.format(self.language, exception)
            logger.error(error_message)
            raise OCRError(error_message)

        return result



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import (
    DocumentCommentCreateView, DocumentCommentDeleteView,
    DocumentCommentListView
)

urlpatterns = patterns(
    '',
    url(
        r'^comment/(?P<pk>\d+)/delete/$', DocumentCommentDeleteView.as_view(),
        name='comment_delete'
    ),
    url(
        r'^(?P<pk>\d+)/comment/add/$', DocumentCommentCreateView.as_view(),
        name='comment_add'
    ),
    url(
        r'^(?P<pk>\d+)/comment/list/$',
        DocumentCommentListView.as_view(), name='comments_for_document'
    ),
)



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _, ungettext

from acls.models import AccessControlList
from common.generics import (
    ConfirmView, SingleObjectEditView, SingleObjectListView
)
from common.mixins import MultipleInstanceActionMixin
from documents.models import Document, DocumentType, DocumentVersion
from permissions import Permission

from .forms import DocumentContentForm
from .models import DocumentVersionOCRError
from .permissions import (
    permission_ocr_content_view, permission_ocr_document,
    permission_ocr_document_delete, permission_document_type_ocr_setup
)


class DocumentAllSubmitView(ConfirmView):
    extra_context = {'title': _('Submit all documents for OCR?')}

    def get_post_action_redirect(self):
        return reverse('common:tools_list')

    def view_action(self):
        count = 0
        for document in Document.objects.all():
            document.submit_for_ocr()
            count += 1

        messages.success(
            self.request, _('%d documents added to the OCR queue.') % count
        )


class DocumentSubmitView(ConfirmView):
    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Submit "%s" to the OCR queue?') % self.get_object()
        }

    def get_object(self):
        return Document.objects.get(pk=self.kwargs['pk'])

    def object_action(self, instance):
        try:
            Permission.check_permissions(
                self.request.user, (permission_ocr_document,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_ocr_document, self.request.user, instance
            )

        instance.submit_for_ocr()

    def view_action(self):
        instance = self.get_object()

        self.object_action(instance=instance)

        messages.success(
            self.request,
            _('Document: %(document)s was added to the OCR queue.') % {
                'document': instance
            }
        )


class DocumentSubmitManyView(MultipleInstanceActionMixin, DocumentSubmitView):
    model = Document
    success_message = '%(count)d document submitted to the OCR queue.'
    success_message_plural = '%(count)d documents submitted to the OCR queue.'

    def get_extra_context(self):
        # Override the base class method
        return {
            'title': _('Submit the selected documents to the OCR queue?')
        }


class DocumentTypeSettingsEditView(SingleObjectEditView):
    fields = ('auto_ocr',)
    view_permission = permission_document_type_ocr_setup

    def get_object(self, queryset=None):
        return get_object_or_404(
            DocumentType, pk=self.kwargs['pk']
        ).ocr_settings

    def get_extra_context(self):
        return {
            'title': _(
                'Edit OCR settings for document type: %s'
            ) % self.get_object().document_type
        }


def document_content(request, document_id):
    document = get_object_or_404(Document, pk=document_id)

    try:
        Permission.check_permissions(
            request.user, (permission_ocr_content_view,)
        )
    except PermissionDenied:
        AccessControlList.objects.check_access(
            permission_ocr_content_view, request.user, document
        )

    document.add_as_recent_document_for_user(request.user)

    content_form = DocumentContentForm(document=document)

    return render_to_response('appearance/generic_form.html', {
        'document': document,
        'form': content_form,
        'hide_labels': True,
        'object': document,
        'read_only': True,
        'title': _('Content of document: %s') % document,
    }, context_instance=RequestContext(request))


class EntryListView(SingleObjectListView):
    extra_context = {
        'hide_object': True,
        'title': _('OCR errors'),
    }
    view_permission = permission_ocr_document

    def get_queryset(self):
        return DocumentVersionOCRError.objects.all()


def entry_delete(request, pk=None, pk_list=None):
    Permission.check_permissions(
        request.user, (permission_ocr_document_delete,)
    )

    if pk:
        entries = [get_object_or_404(DocumentVersionOCRError, pk=pk)]
    elif pk_list:
        entries = [
            get_object_or_404(
                DocumentVersionOCRError, pk=pk
            ) for pk in pk_list.split(',')
        ]
    else:
        messages.error(request, _('Make at least one selection.'))
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        for entry in entries:
            try:
                entry.delete()
                messages.success(
                    request, _('Entry: %(entry)s deleted successfully.') % {
                        'entry': entry
                    }
                )

            except Exception as exception:
                messages.error(
                    request, _('Error entry: %(entry)s; %(error)s') % {
                        'entry': entry, 'error': exception
                    }
                )
        return HttpResponseRedirect(next)

    context = {
        'next': next,
        'previous': previous,
        'delete_view': True,
    }

    if len(entries) == 1:
        context['object'] = entries[0]

    context['title'] = ungettext(
        'Delete the selected entry?',
        'Delete the selected entries?',
        len(entries)
    )

    return render_to_response(
        'appearance/generic_confirm.html', context,
        context_instance=RequestContext(request)
    )


def entry_delete_multiple(request):
    return entry_delete(request, pk_list=request.GET.get('id_list', ''))


def entry_re_queue(request, pk=None, pk_list=None):
    Permission.check_permissions(request.user, (permission_ocr_document,))

    if pk:
        entries = [get_object_or_404(DocumentVersionOCRError, pk=pk)]
    elif pk_list:
        entries = [get_object_or_404(DocumentVersionOCRError, pk=pk) for pk in pk_list.split(',')]
    else:
        messages.error(request, _('Make at least one selection.'))
        return HttpResponseRedirect(request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)))

    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        for entry in entries:
            try:
                entry.document_version.submit_for_ocr()
                messages.success(
                    request,
                    _('Entry: %(entry)s was re-queued for OCR.') % {
                        'entry': entry
                    }
                )
            except DocumentVersion.DoesNotExist:
                messages.error(
                    request, _(
                        'Document version id#: %d, no longer exists.'
                    ) % entry.document_version_id
                )
        return HttpResponseRedirect(next)

    context = {
        'next': next,
        'previous': previous,
    }

    if len(entries) == 1:
        context['object'] = entries[0]

    context['title'] = ungettext(
        'Re-queue the selected entry?',
        'Re-queue the selected entries?',
        len(entries)
    )

    return render_to_response(
        'appearance/generic_confirm.html', context,
        context_instance=RequestContext(request)
    )


def entry_re_queue_multiple(request):
    return entry_re_queue(request, pk_list=request.GET.get('id_list', []))



from __future__ import absolute_import, unicode_literals

from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from documents.models import Document
from documents.views import DocumentListView

from acls.models import AccessControlList
from common.generics import ConfirmView, SingleObjectCreateView
from common.utils import encapsulate, render_date_object
from permissions import Permission

from .exceptions import DocumentAlreadyCheckedOut, DocumentNotCheckedOut
from .forms import DocumentCheckoutForm
from .literals import STATE_LABELS
from .models import DocumentCheckout
from .permissions import (
    permission_document_checkin, permission_document_checkin_override,
    permission_document_checkout
)


class CheckoutDocumentView(SingleObjectCreateView):
    form_class = DocumentCheckoutForm

    def dispatch(self, request, *args, **kwargs):
        self.document = get_object_or_404(Document, pk=self.kwargs['pk'])

        try:
            Permission.check_permissions(
                request.user, (permission_document_checkout,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_checkout, request.user, self.document
            )

        return super(
            CheckoutDocumentView, self
        ).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        try:
            instance = form.save(commit=False)
            instance.user = self.request.user
            instance.document = self.document
            instance.save()
        except DocumentAlreadyCheckedOut:
            messages.error(self.request, _('Document already checked out.'))
        except Exception as exception:
            messages.error(
                self.request,
                _('Error trying to check out document; %s') % exception
            )
        else:
            messages.success(
                self.request,
                _('Document "%s" checked out successfully.') % self.document
            )

        return HttpResponseRedirect(self.get_success_url())

    def get_extra_context(self):
        return {
            'object': self.document,
            'title': _('Check out document: %s') % self.document
        }

    def get_post_action_redirect(self):
        return reverse('checkouts:checkout_info', args=(self.document.pk,))


class CheckoutListView(DocumentListView):
    extra_context = {
        'title': _('Documents checked out'),
        'hide_links': True,
        'extra_columns': (
            {
                'name': _('User'),
                'attribute': encapsulate(
                    lambda document: document.checkout_info().user.get_full_name() or document.checkout_info().user
                )
            },
            {
                'name': _('Checkout time and date'),
                'attribute': encapsulate(
                    lambda document: document.checkout_info().checkout_datetime
                )
            },
            {
                'name': _('Checkout expiration'),
                'attribute': encapsulate(
                    lambda document: document.checkout_info().expiration_datetime
                )
            },
        ),
    }

    def get_document_queryset(self):
        return DocumentCheckout.objects.checked_out_documents()


def checkout_info(request, document_pk):
    document = get_object_or_404(Document, pk=document_pk)
    try:
        Permission.check_permissions(
            request.user, (
                permission_document_checkout, permission_document_checkin
            )
        )
    except PermissionDenied:
        AccessControlList.objects.check_access(
            (permission_document_checkout, permission_document_checkin),
            request.user, document
        )

    paragraphs = [
        _('Document status: %s') % STATE_LABELS[document.checkout_state()]
    ]

    if document.is_checked_out():
        checkout_info = document.checkout_info()
        paragraphs.append(
            _('User: %s') % (
                checkout_info.user.get_full_name() or checkout_info.user
            )
        )
        paragraphs.append(
            _(
                'Check out time: %s'
            ) % render_date_object(checkout_info.checkout_datetime)
        )
        paragraphs.append(
            _(
                'Check out expiration: %s'
            ) % render_date_object(checkout_info.expiration_datetime)
        )
        paragraphs.append(
            _(
                'New versions allowed: %s'
            ) % (_('Yes') if not checkout_info.block_new_version else _('No'))
        )

    return render_to_response(
        'appearance/generic_template.html', {
            'paragraphs': paragraphs,
            'object': document,
            'title': _('Check out details for document: %s') % document
        },
        context_instance=RequestContext(request)
    )


class DocumentCheckinView(ConfirmView):
    def get_extra_context(self):
        document = self.get_object()

        context = {
            'object': document,
        }

        if document.checkout_info().user != self.request.user:
            context['title'] = _(
                'You didn\'t originally checked out this document. '
                'Forcefully check in the document: %s?'
            ) % document
        else:
            context['title'] = _('Check in the document: %s?') % document

        return context

    def get_object(self):
        return get_object_or_404(Document, pk=self.kwargs['pk'])

    def get_post_action_redirect(self):
        return reverse('checkouts:checkout_info', args=(self.get_object().pk,))

    def view_action(self, request):
        document = self.get_object()

        if document.checkout_info().user == request.user:
            try:
                Permission.check_permissions(
                    request.user, (permission_document_checkin,)
                )
            except PermissionDenied:
                AccessControlList.objects.check_access(
                    permission_document_checkin, request.user, document
                )
        else:
            try:
                Permission.check_permissions(
                    request.user, (permission_document_checkin_override,)
                )
            except PermissionDenied:
                AccessControlList.objects.check_access(
                    permission_document_checkin_override, request.user,
                    document
                )

        try:
            document.check_in(user=request.user)
        except DocumentNotCheckedOut:
            messages.error(request, _('Document has not been checked out.'))
        except Exception as exception:
            messages.error(
                request, _('Error trying to check in document; %s') % exception
            )
        else:
            messages.success(
                request, _('Document "%s" checked in successfully.') % document
            )
